"""Tests for config file"""
import unittest
import os
from neural_compressor.conf import config as conf


def helper(content):
    with open('fake_conf.yaml', 'w', encoding="utf-8") as f:
        f.write(content)


class TestConf(unittest.TestCase):
    @classmethod
    def tearDownClass(self):
        os.remove('fake_conf.yaml')

    def test_main_key(self):
        test = '''
        model:
          name: main_key_yaml
          framework: pytorch
        test: cpu
        '''
        helper(test)
        self.assertRaises(RuntimeError, conf.Conf, 'fake_conf.yaml')

    def test_framework(self):
        test = '''
        model:
          name: framework_yaml 
          framework: pytorch, mxnet
        '''
        helper(test)
        self.assertRaises(RuntimeError, conf.Conf, 'fake_conf.yaml')

        test = '''
        device: cpu
        '''
        helper(test)
        self.assertRaises(RuntimeError, conf.Conf, 'fake_conf.yaml')

    def test_device(self):
        test = '''
        model:
          name: device_yaml 
          framework: mxnet
        device: xpu
        '''
        helper(test)
        self.assertRaises(RuntimeError, conf.Conf, 'fake_conf.yaml')

        test = '''
        model:
          name: device_yaml 
          framework: mxnet
        device: cpu, gpu
        '''
        helper(test)
        self.assertRaises(RuntimeError, conf.Conf, 'fake_conf.yaml')

    def test_version(self):
        test = '''
        version: 2.0

        model:
          name: version_yaml 
          framework: mxnet
        '''
        helper(test)
        self.assertRaises(RuntimeError, conf.Conf, 'fake_conf.yaml')

        test = '''
        model:
          name: version_yaml 
          framework: mxnet
        '''
        helper(test)
        config = conf.Conf('fake_conf.yaml')
        self.assertEqual(config.usr_cfg.version, 1.0)

        test = '''
        version: 1.0

        model:
          name: version_yaml 
          framework: mxnet
        '''
        helper(test)
        config = conf.Conf('fake_conf.yaml')
        self.assertEqual(config.usr_cfg.version, 1.0)

    def test_calibration(self):
        test = '''
        model:
          name: calib_yaml 
          framework: mxnet
        quantization:
          calibration:
            sampling_sizes: 10
        '''
        helper(test)
        self.assertRaises(RuntimeError, conf.Conf, 'fake_conf.yaml')

        test = '''
        model:
          name: calib_yaml 
          framework: mxnet
        quantization:
          calibration:
            sampling_size:
        '''
        helper(test)
        self.assertRaises(RuntimeError, conf.Conf, 'fake_conf.yaml')

        test = '''
        model:
          name: calib_yaml 
          framework: mxnet
        quantization:
          calibration:
            dataloader:
        '''
        helper(test)
        self.assertRaises(RuntimeError, conf.Conf, 'fake_conf.yaml')

        test = '''
        model:
          name: calib_yaml 
          framework: mxnet
        quantization:
          calibration:
          op_wise: {
            'test': {
                'activation': [{'dtype': 'uint8'}, {'algorithm': 'minmax'}]
            }
          }

        '''
        helper(test)
        self.assertRaises(RuntimeError, conf.Conf, 'fake_conf.yaml')

    def test_quantization(self):
        test = '''
        model:
          name: quant_yaml 
          framework: mxnet
        quantization:
          model_wise:
            weights:
            granularity: per_channel
        '''
        helper(test)
        self.assertRaises(RuntimeError, conf.Conf, 'fake_conf.yaml')

        test = '''
        model:
          name: quant_yaml 
          framework: mxnet
        quantization:
          model_wise:
          approach:
        '''
        helper(test)
        self.assertRaises(RuntimeError, conf.Conf, 'fake_conf.yaml')

        test = '''
        model:
          name: quant_yaml 
          framework: mxnet
        quantization:
          approach: post_training_static_quant, quant_aware_training
        '''
        helper(test)
        self.assertRaises(RuntimeError, conf.Conf, 'fake_conf.yaml')

        test = '''
        model:
          name: quant_yaml 
          framework: mxnet
        quantization:
          model_wise:
            activation:
              scheme: asym
              dtype: int8
            weight:
              scheme: asym
              dtype: int8
        '''
        helper(test)
        conf.Conf('fake_conf.yaml')

        test = '''
        model:
          name: quant_yaml 
          framework: mxnet
        quantization:
          model_wise:
            activation:
              scheme:
              dtype: int8
            weight:
              scheme: asym
              dtype: int8
        '''
        helper(test)
        self.assertRaises(RuntimeError, conf.Conf, 'fake_conf.yaml')

    def test_tuning(self):
        test = '''
        model:
          name: tuning_yaml 
          framework: mxnet
        tuning:
          accuracy_criterion:
            relative: 0.01
          strategy:
            name: basic, mse
        '''
        helper(test)
        self.assertRaises(RuntimeError, conf.Conf, 'fake_conf.yaml')

        test = '''
        model:
          name: tuning_yaml 
          framework: mxnet
        tuning:
          accuracy_criterion:
          relative: 0.01
        '''
        helper(test)
        self.assertRaises(RuntimeError, conf.Conf, 'fake_conf.yaml')

        test = '''
        model:
          name: tuning_yaml 
          framework: mxnet
        tuning:
          accuracy_criterion:
          relative: 0.01
        '''
        helper(test)
        self.assertRaises(RuntimeError, conf.Conf, 'fake_conf.yaml')

        test = '''
        model:
          name: tuning_yaml 
          framework: mxnet
        tuning:
          accuracy_criterion:
            relative: 0.01
          strategy:
            name: fake
        '''
        helper(test)
        self.assertRaises(RuntimeError, conf.Conf, 'fake_conf.yaml')

        test = '''
        model:
          name: tuning_yaml 
          framework: mxnet
        tuning:
          accuracy_criterion:
            relative:
          strategy:
            name: basic
        '''
        helper(test)
        self.assertRaises(RuntimeError, conf.Conf, 'fake_conf.yaml')

        test = '''
        model:
          name: tuning_yaml 
          framework: mxnet
        tuning:
          accuracy_criterion:
          exit_policy:
            timeout: 3
        '''
        helper(test)
        self.assertRaises(RuntimeError, conf.Conf, 'fake_conf.yaml')

        test = '''
        model:
          name: tuning_yaml 
          framework: mxnet
        tuning:
          accuracy_criterion:
            relative: 0.01
            absolute: 0.01
        '''
        helper(test)
        self.assertRaises(RuntimeError, conf.Conf, 'fake_conf.yaml')

    def test_workspace(self):
        test = '''
        model:
          name: workspace_yaml 
          framework: mxnet
        tuning:
          workspace: 
            -path: ./workspace
        '''
        helper(test)
        self.assertRaises(RuntimeError, conf.Conf, 'fake_conf.yaml')

    def test_inputs_outputs(self):
        test = '''
        model:
          name: inout_yaml 
          framework: mxnet
          inputs: x, y
        '''
        helper(test)
        config = conf.Conf('fake_conf.yaml')
        self.assertEqual(config.usr_cfg.model.inputs, ['x', 'y'])

    def test_modelwise_conf_merge(self):
        test = '''
        model:
          name: inout_yaml 
          framework: mxnet
        quantization:
          model_wise:
            weight:
              algorithm:  minmax
            activation:
              algorithm:  minmax
        '''
        helper(test)
        config = conf.Quantization_Conf('fake_conf.yaml')

        framework_modelwise_capability = {
            'CONV2D': {
                'activation': {
                    'dtype': ['uint8', 'fp32'],
                    'scheme': ['asym', 'sym'],
                    'granularity': ['per_tensor'],
                    'algorithm': ['minmax', 'kl']
                },
                'weight': {
                    'dtype': ['int8', 'fp32'],
                    'scheme': [
                        'sym',
                    ],
                    'granularity': ['per_channel', 'per_tensor'],
                    'algorithm': ['minmax']
                },
            },
        }

        tune_space = config.modelwise_tune_space(framework_modelwise_capability)
        self.assertEqual(tune_space['CONV2D']['activation']['algorithm'], ['minmax'])

    def test_modelwise_conf_merge2(self):
        test = '''
        model:
          name: inout_yaml 
          framework: mxnet
        quantization:
          model_wise:
            weight:
              algorithm:  minmax
            activation:
              algorithm:  minmax
              dtype: ['uint8', 'fp32']
        '''
        helper(test)
        config = conf.Quantization_Conf('fake_conf.yaml')

        framework_modelwise_capability = {
            'CONV2D': {
                'activation': {
                    'dtype': ['iint8', 'fp32'],
                    'scheme': ['asym', 'sym'],
                    'granularity': ['per_tensor'],
                    'algorithm': ['minmax', 'kl']
                },
                'weight': {
                    'dtype': ['int8', 'fp32'],
                    'scheme': [
                        'sym',
                    ],
                    'granularity': ['per_channel', 'per_tensor'],
                    'algorithm': ['minmax']
                },
            },
        }

        tune_space = config.modelwise_tune_space(framework_modelwise_capability)
        self.assertEqual(tune_space['CONV2D']['activation']['dtype'], ['fp32'])
  
    def test_ops_override(self):
        test = '''
        model:
          name: ops_override_yaml 
          framework: mxnet
        quantization:
          op_wise: {
            'conv1': {
              'activation':  {'dtype': ['uint8', 'fp32'], 'algorithm': ['minmax'], 'scheme':['sym']},
              'weight': {'dtype': ['int8', 'fp32'], 'algorithm': ['minmax']}
            },
            'conv2': {
              'activation':  {'dtype': ['fp32']},
              'weight': {'dtype': ['fp32']}
            }
          }
        tuning:
          accuracy_criterion:
            relative: 0.01
          objective: performance
          
        '''
        helper(test)
        config = conf.Quantization_Conf('fake_conf.yaml')

        framework_modelwise_capability = {
            'CONV2D': {
                'activation': {
                    'dtype': ['uint8', 'fp32'],
                    'scheme': ['asym', 'sym'],
                    'granularity': ['per_tensor'],
                    'algorithm': ['minmax', 'kl']
                },
                'weight': {
                    'dtype': ['int8', 'fp32'],
                    'scheme': [
                        'sym',
                    ],
                    'granularity': ['per_channel', 'per_tensor'],
                    'algorithm': ['minmax']
                },
            },
        }

        config.modelwise_tune_space(framework_modelwise_capability)

        framework_opwise_capability = {
            ('conv1', 'CONV2D'): {
                'activation': {
                    'dtype': ['uint8', 'fp32'],
                    'scheme': ['asym', 'sym'],
                    'granularity': ['per_tensor'],
                    'algorithm': ['minmax', 'kl']
                },
                'weight': {
                    'dtype': ['int8', 'fp32'],
                    'scheme': [
                        'sym',
                    ],
                    'granularity': ['per_channel', 'per_tensor'],
                    'algorithm': ['minmax']
                }},
            ('conv2', 'CONV2D'): {
                'activation': {
                    'dtype': ['uint8', 'fp32'],
                    'scheme': ['asym', 'sym'],
                    'granularity': ['per_tensor'],
                    'algorithm': ['minmax', 'kl']
                },
                'weight': {
                    'dtype': ['int8', 'fp32'],
                    'scheme': [
                        'sym',
                    ],
                    'granularity': ['per_channel', 'per_tensor'],
                    'algorithm': ['minmax']
                }},
        }

        tune_space = config.opwise_tune_space(framework_opwise_capability)
        self.assertEqual(tune_space[('conv1', 'CONV2D')]['weight']['algorithm'], ['minmax'])
        self.assertEqual(tune_space[('conv2', 'CONV2D')]['activation']['dtype'], ['fp32'])
  
    def test_prune(self):
        test = '''
        model:
          name: imagenet_prune
          framework: pytorch
        
        pruning:
          train:
            start_epoch: 0
            end_epoch: 4
            dataloader:
              batch_size: 30
              dataset:
                ImageFolder:
                  root: /path/to/training/dataset
            optimizer:
              SGD:
                learning_rate: 0.1
                momentum: 0.1   
                nesterov: True
                weight_decay: 0.1     
            criterion:
              CrossEntropyLoss:
                reduction: sum
          approach:
            weight_compression:
              initial_sparsity: 0.0
              target_sparsity: 0.97
              pruners:
                - !Pruner
                    start_epoch: 1
                    end_epoch: 3
                    names: ['layer1.0.conv1.weight']
        
                - !Pruner
                    start_epoch: 0
                    end_epoch: 4
                    target_sparsity: 0.6
                    update_frequency: 2
                    names: ['layer1.0.conv2.weight']
        '''
        helper(test)
        config = conf.Conf('fake_conf.yaml')

    def test_data_type(self):
        test = '''
        model:
          name: test
          framework: tensorflow

        quantization:
          calibration:
            sampling_size: 20
            dataloader:
              batch_size: 1
              dataset:
                dummy:
                  shape: [[224,224], [256,256]]
                  high: [128., 127]
                  low: 1
                  dtype: ['float32', 'int8']
        '''
        helper(test)
        cfg = conf.Conf('fake_conf.yaml').usr_cfg
        dataset = cfg['quantization']['calibration']['dataloader']['dataset']['dummy']
        self.assertTrue(isinstance(dataset['shape'][0], tuple))
        self.assertTrue(isinstance(dataset['shape'], list))
        self.assertTrue(isinstance(dataset['high'][1], float))
        self.assertTrue(isinstance(dataset['high'][0], float))
        self.assertTrue(isinstance(dataset['low'], float))

        test = '''
        model:
          name: test
          framework: tensorflow

        quantization:
          calibration:
            sampling_size: 20
            dataloader:
              batch_size: 1
              dataset:
                dummy:
                  shape: [224,224]
                  high: 128
                  low: 0.1
                  dtype: ['float32', 'int8']
        '''
        helper(test)
        cfg = conf.Conf('fake_conf.yaml').usr_cfg
        dataset = cfg['quantization']['calibration']['dataloader']['dataset']['dummy']
        self.assertTrue(isinstance(dataset['shape'], tuple))
        self.assertTrue(isinstance(dataset['high'], float)) 

        test = '''
        model:
          name: test
          framework: tensorflow

        quantization:
          calibration:
            sampling_size: 20
            dataloader:
              batch_size: 1
              dataset:
                style_transfer:
                  content_folder: test
                  style_folder: test
                  crop_ratio: 0.5
                  resize_shape: 10,10
              transform:
                RandomResizedCrop:
                  size: 10
                  scale: [0.07, 0.99]
                  ratio: [0.6, 0.8]
        '''
        helper(test)
        cfg = conf.Conf('fake_conf.yaml').usr_cfg
        shape_cfg = cfg['quantization']['calibration']['dataloader']['dataset']['style_transfer']['resize_shape']
        self.assertTrue(isinstance(shape_cfg, list)) 
        transform_cfg = cfg['quantization']['calibration']['dataloader']['transform']['RandomResizedCrop']
        self.assertTrue(isinstance(transform_cfg['scale'], list))
        self.assertTrue(isinstance(transform_cfg['ratio'], list))

        test = '''
        model:
          name: test
          framework: tensorflow

        quantization:
          calibration:
            sampling_size: 20
            dataloader:
              batch_size: 1
              dataset:
                style_transfer:
                  content_folder: test
                  style_folder: test
                  crop_ratio: 0.5
                  resize_shape: [10,10]
        '''
        helper(test)
        cfg = conf.Conf('fake_conf.yaml').usr_cfg
        shape_cfg = cfg['quantization']['calibration']['dataloader']['dataset']['style_transfer']['resize_shape']
        self.assertTrue(isinstance(shape_cfg, list)) 

        test = '''
        model:
          name: test
          framework: tensorflow

        quantization:
          calibration:
            sampling_size: 20
            dataloader:
              batch_size: 1
              dataset:
                dummy:
                  shape: [224,224]
              transform:
                BilinearImagenet:
                  height: 224
                  width: 224
                  mean_value: 123.68 116.78 103.94
        '''
        helper(test)
        cfg = conf.Conf('fake_conf.yaml').usr_cfg
        shape_cfg = cfg['quantization']['calibration']['dataloader']['dataset']['dummy']['shape']
        self.assertTrue(isinstance(shape_cfg, tuple)) 
        transform_cfg = cfg['quantization']['calibration']['dataloader']['transform']['BilinearImagenet']
        self.assertTrue(isinstance(transform_cfg['mean_value'], list))


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2021 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from .tf_extractor import TensorflowExtractor
from .onnx_extractor import ONNXExtractor
from neural_compressor.utils import logger


EXTRACTORS = {
    'tensorflow': TensorflowExtractor,
    'onnxruntime': ONNXExtractor,
}


class Extractor(object):
    """
    A super class for an operation extractor.
    Do additional extraction of operation attributes without modifying of graph topology.
    """

    def __call__(self, model):
        # framework = model.framework_specific_info['framework']
        framework = model.framework()
        extractor = EXTRACTORS[framework]()
        model = extractor(model)
        logger.info('Extract {} model done...'.format(framework))
        return model



"""Tests for neural_compressor quantization"""
import unittest
import os
import importlib
import shutil
import yaml
import numpy as np
import tensorflow as tf

def build_fake_yaml_footprint():
    fake_yaml = '''
        model:
          name: fake_yaml
          framework: tensorflow
          inputs: x
          outputs: op_to_store
        device: cpu
        evaluation:
          accuracy:
            metric:
              topk: 1
          performance: {}
        tuning:
          objective: footprint
          strategy:
            name: fake
          accuracy_criterion:
            relative: 0.01
          workspace:
            path: saved
        '''
    y = yaml.load(fake_yaml, Loader=yaml.SafeLoader)
    with open('fake_yaml_footprint.yaml', "w", encoding="utf-8") as f:
        yaml.dump(y, f)
    f.close()

def build_fake_yaml_model_size():
    fake_yaml = '''
        model:
          name: fake_yaml
          framework: tensorflow
          inputs: x
          outputs: op_to_store
        device: cpu
        evaluation:
          accuracy:
            metric:
              topk: 1
          performance: {}
        tuning:
          objective: modelsize
          strategy:
            name: fake
          accuracy_criterion:
            relative: 0.01
          workspace:
            path: saved
        '''
    y = yaml.load(fake_yaml, Loader=yaml.SafeLoader)
    with open('fake_yaml_model_size.yaml', "w", encoding="utf-8") as f:
        yaml.dump(y, f)
    f.close()

def build_fake_yaml():
    fake_yaml = '''
        model:
          name: fake_yaml
          framework: tensorflow
          inputs: x
          outputs: op_to_store
        device: cpu
        evaluation:
          accuracy:
            metric:
              topk: 1
          performance: {}
        tuning:
          objective: performance
          strategy:
            name: fake
          accuracy_criterion:
            relative: 0.01
          workspace:
            path: saved
        '''
    y = yaml.load(fake_yaml, Loader=yaml.SafeLoader)
    with open('fake_yaml.yaml', "w", encoding="utf-8") as f:
        yaml.dump(y, f)
    f.close()

def build_fake_model():
    try:
        graph = tf.Graph()
        graph_def = tf.GraphDef()
        with tf.Session(graph=graph) as sess:
            x = tf.placeholder(tf.float64, shape=(1, 256, 256, 1), name='x')
            y = tf.constant(np.random.random((2, 2, 1, 1)), name='y')
            op = tf.nn.conv2d(input=x, filter=y, strides=[1, 1, 1, 1], \
                              padding='VALID', name='op_to_store')

            sess.run(tf.global_variables_initializer())
            constant_graph = tf.graph_util.convert_variables_to_constants(sess, sess.graph_def, ['op_to_store'])

        graph_def.ParseFromString(constant_graph.SerializeToString())
        with graph.as_default():
            tf.import_graph_def(graph_def, name='')
    except:
        graph = tf.Graph()
        graph_def = tf.compat.v1.GraphDef()
        with tf.compat.v1.Session(graph=graph) as sess:
            x = tf.compat.v1.placeholder(tf.float64, shape=(1, 256, 256, 1), name='x')
            y = tf.compat.v1.constant(np.random.random((3, 3, 1, 1)), name='y')
            op = tf.nn.conv2d(input=x, filters=y, strides=[1, 1, 1, 1], \
                              padding='VALID', name='op_to_store')

            sess.run(tf.compat.v1.global_variables_initializer())
            constant_graph = tf.compat.v1.graph_util.convert_variables_to_constants(sess, sess.graph_def, ['op_to_store'])

        graph_def.ParseFromString(constant_graph.SerializeToString())
        with graph.as_default():
            tf.import_graph_def(graph_def, name='')
    return graph

def build_fake_model1():
    try:
        graph = tf.Graph()
        graph_def = tf.GraphDef()
        with tf.Session(graph=graph) as sess:
            x = tf.placeholder(tf.float64, shape=(1, 256, 256, 1), name='x')
            y_1 = tf.constant(np.random.random((3, 3, 1, 1)), name='y_1')
            y_2 = tf.constant(np.random.random((3, 3, 1, 1)), name='y_2')
            conv1 = tf.nn.conv2d(input=x, filter=y_1, strides=[1, 1, 1, 1], \
                                 padding='VALID', name='conv1')
            op = tf.nn.conv2d(input=conv1, filter=y_2, strides=[1, 1, 1, 1], \
                              padding='VALID', name='op_to_store')

            sess.run(tf.global_variables_initializer())
            constant_graph = tf.graph_util.convert_variables_to_constants(sess, sess.graph_def, ['op_to_store'])

        graph_def.ParseFromString(constant_graph.SerializeToString())
        with graph.as_default():
            tf.import_graph_def(graph_def, name='')
    except:
        graph = tf.Graph()
        graph_def = tf.compat.v1.GraphDef()
        with tf.compat.v1.Session(graph=graph) as sess:
            x = tf.compat.v1.placeholder(tf.float64, shape=(1, 256, 256, 1), name='x')
            y_1 = tf.constant(np.random.random((3, 3, 1, 1)), name='y_1')
            y_2 = tf.constant(np.random.random((3, 3, 1, 1)), name='y_2')
            conv1 = tf.nn.conv2d(input=x, filters=y_1, strides=[1, 1, 1, 1], \
                                 padding='VALID', name='conv1')
            op = tf.nn.conv2d(input=conv1, filters=y_2, strides=[1, 1, 1, 1], \
                              padding='VALID', name='op_to_store')

            sess.run(tf.compat.v1.global_variables_initializer())
            constant_graph = tf.compat.v1.graph_util.convert_variables_to_constants(sess, sess.graph_def, ['op_to_store'])

        graph_def.ParseFromString(constant_graph.SerializeToString())
        with graph.as_default():
            tf.import_graph_def(graph_def, name='')
    return graph

def build_fake_strategy():
    with open(os.path.join(os.path.dirname(importlib.util.find_spec('neural_compressor').origin), \
              'strategy/fake.py'), 'w', encoding='utf-8') as f:
        seq = [
            "import time\n",
            "from .strategy import strategy_registry, TuneStrategy\n",
            "from collections import OrderedDict\n",
            "import copy\n",
            "@strategy_registry\n",
            "class FakeTuneStrategy(TuneStrategy):\n",
            "  def __init__(self, model, cfg, q_dataloader, q_func=None, eval_dataloader=None, eval_func=None, dicts=None, q_hooks=None):\n",
            "    self.id = 0\n",
            "    self.resume = True if dicts else False\n",
            "    super(FakeTuneStrategy, self).__init__(model, cfg, q_dataloader, q_func, eval_dataloader, eval_func, dicts)\n",
            "  def __getstate__(self):\n",
            "    for history in self.tuning_history:\n",
            "      if self._same_yaml(history['cfg'], self.cfg):\n",
            "        history['id'] = self.id\n",
            "    save_dict = super(FakeTuneStrategy, self).__getstate__()\n",
            "    return save_dict\n",
            "  def next_tune_cfg(self):\n",
            "    if self.resume:\n",
            "      assert self.id == 1\n",
            "      assert len(self.tuning_history) == 1\n",
            "      history = self.tuning_history[0]\n",
            "      assert self._same_yaml(history['cfg'], self.cfg)\n",
            "      assert len(history['history'])\n",
            "      for h in history['history']:\n",
            "        assert h\n",
            "    op_cfgs = {}\n",
            "    for iterations in self.calib_iter:\n",
            "      op_cfgs['calib_iteration'] = int(iterations)\n",
            "      op_cfgs['op'] = OrderedDict()\n",
            "      for op in self.opwise_quant_cfgs:\n",
            "        op_cfgs['op'][op] = copy.deepcopy(\n",
            "                                self.opwise_tune_cfgs[op][0])\n",
            "      self.id += 1\n",
            "      yield op_cfgs\n",
            "      return\n"
        ]
        f.writelines(seq)
    f.close()

class TestObjective(unittest.TestCase):
    @classmethod
    def setUpClass(self):
        self.constant_graph = build_fake_model()
        self.constant_graph_1 = build_fake_model1()
        build_fake_yaml()
        build_fake_yaml_footprint()
        build_fake_yaml_model_size()
        build_fake_strategy()

    @classmethod
    def tearDownClass(self):
        os.remove('fake_yaml.yaml')
        os.remove('fake_yaml_model_size.yaml')
        os.remove('fake_yaml_footprint.yaml')
        os.remove(os.path.join(os.path.dirname(importlib.util.find_spec('neural_compressor').origin), 'strategy/fake.py'))
        shutil.rmtree('./saved', ignore_errors=True)

    def test_performance(self):
        from neural_compressor.data import DATASETS
        dataset = DATASETS('tensorflow')['dummy']((100, 256, 256, 1), label=True)

        from neural_compressor.experimental import Quantization, common
        from neural_compressor.utils.utility import get_size

        quantizer = Quantization('fake_yaml.yaml')
        quantizer.calib_dataloader = common.DataLoader(dataset)
        quantizer.eval_dataloader = common.DataLoader(dataset)
        quantizer.model = self.constant_graph
        q_model = quantizer()

        from neural_compressor.experimental import Benchmark, common
        benchmarker = Benchmark('fake_yaml.yaml')
        benchmarker.b_dataloader = common.DataLoader(dataset)
        benchmarker.model = self.constant_graph_1
        benchmarker()

    def test_model_size(self):
        from neural_compressor.experimental import Benchmark, common
        from neural_compressor.data import DATASETS
        dataset = DATASETS('tensorflow')['dummy']((100, 256, 256, 1), label=True)

        benchmarker = Benchmark('fake_yaml_model_size.yaml')
        benchmarker.b_dataloader = common.DataLoader(dataset)
        benchmarker.model = self.constant_graph_1
        benchmarker(mode='accuracy')

    def test_footprint(self):
        from neural_compressor.experimental import Benchmark, common
        from neural_compressor.data import DATASETS
        dataset = DATASETS('tensorflow')['dummy']((100, 256, 256, 1), label=True)

        benchmarker = Benchmark('fake_yaml_footprint.yaml')
        benchmarker.b_dataloader = common.DataLoader(dataset)
        benchmarker.model = self.constant_graph_1
        benchmarker()


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2021 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from .op import Operator, operator_registry
from .tensor import Tensor
import copy


@operator_registry(operator_type='OneHot')
class OneHot(Operator):
    def __init__(self):
        super().__init__()

    def set_attr(self, framework, node):
        if framework == 'tensorflow':
            self._attr['axis'] = node.attr['axis'].i
            self._attr['depth'] = int(self._input_tensors[1].data)
            self._attr['on_value'] = int(self._input_tensors[2].data)  # float
            self._attr['off_value'] = int(self._input_tensors[3].data)  # float
            self._input_tensors = [copy.deepcopy(self._input_tensors[0])]



import random
import copy

import unittest
import numpy as np

from neural_compressor.experimental.pruning_recipes.patterns import patterns

class TestPruningPattern(unittest.TestCase):

    tensor_4d = np.random.random([1,100,2240,2240])
    tensor_2d = np.random.random([5120,2560])

    def test_tile_pattern(self):
        for tensor in [self.tensor_2d, self.tensor_4d]:
            shape = list(tensor.shape)

            pattern_1x1 = patterns['tile_pattern_1x1']()
            shape_1x1 = shape[:-2] + [shape[-2] / 1] + [shape[-1] / 1]
            sparse_tensor = self.sparsify_tensor(tensor, [1,1], 0.8)
            self.assertEqual(list(pattern_1x1.reduce(sparse_tensor).shape), shape_1x1)
            self.assertAlmostEqual(pattern_1x1.compute_sparsity(sparse_tensor), 0.8, delta=0.01)

            pattern_2x2 = patterns['tile_pattern_2x2']()
            shape_2x2 = shape[:-2] + [shape[-2] / 2] + [shape[-1] / 2]
            sparse_tensor = self.sparsify_tensor(tensor, [2,2], 0.7)
            self.assertEqual(list(pattern_2x2.reduce(sparse_tensor).shape), shape_2x2)
            self.assertAlmostEqual(pattern_2x2.compute_sparsity(sparse_tensor), 0.7, delta=0.01)

            pattern_1x16 = patterns['tile_pattern_1x16']()
            shape_1x16 = shape[:-2] + [shape[-2] / 1] + [shape[-1] / 16]
            sparse_tensor = self.sparsify_tensor(tensor, [1,16], 0.5)
            self.assertEqual(list(pattern_1x16.reduce(sparse_tensor).shape), shape_1x16)
            self.assertAlmostEqual(pattern_1x16.compute_sparsity(sparse_tensor), 0.5, delta=0.02)

    def sparsify_tensor(self, tensor, mask_shape, ratio):
        tensor = copy.deepcopy(tensor)
        for i in range(tensor.shape[-2]//mask_shape[-2]):
            for j in range(tensor.shape[-1]//mask_shape[-1]):
                if random.random() < ratio:
                    tensor[..., i*mask_shape[-2]:(i+1)*mask_shape[-2], j*mask_shape[-1]:(j+1)*mask_shape[-1]] = 0
        return tensor


if __name__ == "__main__":
    unittest.main()



import os
import shutil
import unittest

import torch
import torchvision
import yaml
import onnx
import numpy as np
from collections import OrderedDict
from onnx import onnx_pb as onnx_proto
from onnx import helper, TensorProto, numpy_helper
from neural_compressor.adaptor import FRAMEWORKS
from neural_compressor.data import DATASETS, DATALOADERS
from neural_compressor.experimental import Quantization, common
from neural_compressor.experimental import Benchmark, common

def build_static_yaml():
    fake_yaml = """
        model:
          name: imagenet
          framework: onnxrt_qlinearops

        quantization:                                        
          approach: post_training_static_quant  
          calibration:
            sampling_size: 50
          op_wise: {
            'Gather_*': {
            'activation':  {'dtype': ['fp32'], 'scheme':['sym']},
            'weight': {'dtype': ['fp32'], 'scheme':['sym']}
            }
          }
 
        evaluation:
          accuracy:
            metric:
              topk: 1

        tuning:
          accuracy_criterion:
            relative:  0.01
          exit_policy:
            timeout: 0
          random_seed: 9527
          workspace: 
            path: ./nc_workspace/recover/
        """
    with open("static.yaml", "w", encoding="utf-8") as f:
        f.write(fake_yaml)

def build_benchmark_yaml():
    fake_yaml = """
        model:
          name: imagenet
          framework: onnxrt_qlinearops

        evaluation:
          performance:
            warmup: 1
            iteration: 10
            configs:
              num_of_instance: 1
            dataloader:
              batch_size: 1
              dataset:
                ImageFolder:
                  root: /path/to/evaluation/dataset/
          accuracy:
            metric:
              topk: 1

        tuning:
          accuracy_criterion:
            relative:  0.01
          exit_policy:
            timeout: 0
          random_seed: 9527
        """
    with open("benchmark.yaml", "w", encoding="utf-8") as f:
        f.write(fake_yaml)

def build_dynamic_yaml():
    fake_yaml = """
        model:
          name: imagenet
          framework: onnxrt_integerops

        quantization:                                        
          approach: post_training_dynamic_quant 
          calibration:
              sampling_size: 50

        evaluation:
          accuracy:
            metric:
              topk: 1

        tuning:
          accuracy_criterion:
            relative:  0.01
          exit_policy:
            timeout: 0
          random_seed: 9527
        """
    with open("dynamic.yaml", "w", encoding="utf-8") as f:
        f.write(fake_yaml)

def build_gather_yaml():
    fake_yaml = """
        model:
          name: imagenet
          framework: onnxrt_qlinearops

        quantization:                                        
          approach: post_training_static_quant 
          calibration:
              sampling_size: 1
        evaluation:
          accuracy:
            metric:
              Accuracy: {}

        tuning:
          accuracy_criterion:
            relative:  -0.01
          exit_policy:
            timeout: 0
          random_seed: 9527
        """
    with open("gather.yaml", "w", encoding="utf-8") as f:
        f.write(fake_yaml)

def build_rename_yaml():
    fake_yaml = """
        model:
          name: test
          framework: onnxrt_integerops

        quantization:                                        
          approach: post_training_dynamic_quant 
          calibration:
              sampling_size: 1

        evaluation:
          accuracy:
            metric:
              Accuracy: {}

        tuning:
          accuracy_criterion:
            relative:  0.01
          exit_policy:
            timeout: 0
          random_seed: 9527
        """
    with open("rename.yaml", "w", encoding="utf-8") as f:
        f.write(fake_yaml)

def build_non_MSE_yaml():
    fake_yaml = """
        model:
          name: imagenet
          framework: onnxrt_qlinearops

        quantization:
          approach: post_training_static_quant
          calibration:
              sampling_size: 50
          op_wise: {
            'Gather_*': {
            'activation':  {'dtype': ['fp32'], 'scheme':['sym']},
            'weight': {'dtype': ['fp32'], 'scheme':['sym']}
            }
          }

        evaluation:
          accuracy:
            metric:
              MSE: 
               compare_label: False
          performance:
            warmup: 5
            iteration: 10

        tuning:
          accuracy_criterion:
            relative:  0.1
          exit_policy:
            timeout: 0
          random_seed: 9527
        """
    with open("non_MSE.yaml", "w", encoding="utf-8") as f:
        f.write(fake_yaml)

def eval_func(model):
    return 1.0

def get_torch_version():
    try:
        torch_version = torch.__version__.split('+')[0]
    except ValueError as e:
        assert False, 'Got an unknow version of torch: {}'.format(e)
    return torch_version

def export_onnx_model(model, path, opset=12):
    x = torch.randn(100, 3, 224, 224, requires_grad=True)
    torch_out = model(x)

    # Export the model
    torch.onnx.export(model,               # model being run
                    x,                         # model input (or a tuple for multiple inputs)
                    path,                      # where to save the model (can be a file or file-like object)
                    export_params=True,        # store the trained parameter weights inside the model file
                    opset_version=opset,          # the ONNX version to export the model to, please ensure at least 11.
                    do_constant_folding=True,  # whether to execute constant folding for optimization
                    input_names = ["input"],   # the model"s input names
                    output_names = ["output"], # the model"s output names
                    dynamic_axes={"input" : {0 : "batch_size"},    # variable length axes
                                  "output" : {0 : "batch_size"}})

def generate_input_initializer(tensor_shape, tensor_dtype, input_name):
    '''
    Helper function to generate initializers for test inputs
    '''
    tensor = np.random.ranf(tensor_shape).astype(tensor_dtype)
    init = numpy_helper.from_array(tensor, input_name)
    return init  

def build_ir3_model():
    input0 = helper.make_tensor_value_info('input0', TensorProto.FLOAT, [1, 2048])
    output = helper.make_tensor_value_info('output', TensorProto.FLOAT, [1, 1000])
    weight = helper.make_tensor_value_info('X1_weight', TensorProto.FLOAT, [1000, 2048])

    X1_weight = generate_input_initializer([1000, 2048], np.float32, 'X1_weight')
    X1_bias = generate_input_initializer([1000], np.float32, 'X1_bias')
    kwargs = {'alpha':1.0, 'beta':1.0, 'transA':0, 'transB':1}
    gemm = helper.make_node('Gemm', ['input0', 'X1_weight'], ['output'], name='gemm', **kwargs)

    graph = helper.make_graph([gemm], 'test_graph_6', [input0], [output])
    graph.initializer.add().CopyFrom(X1_weight)
    graph.initializer.add().CopyFrom(X1_bias)  
    graph.input.extend([weight])
    model = helper.make_model(graph)
    model = helper.make_model(graph, **{'opset_imports': [helper.make_opsetid('', 13)]})
    model.ir_version = 3
    return model

def build_matmul_model():
    A = helper.make_tensor_value_info('A', TensorProto.FLOAT, [1, 1, 5, 5])
    B = helper.make_tensor_value_info('B', TensorProto.FLOAT, [1, 1, 5, 1])
    C = helper.make_tensor_value_info('C', TensorProto.FLOAT, [1, 1, 5, 1])
    matmul_node = onnx.helper.make_node('MatMul', ['A', 'B'], ['C'], name='Matmul')
    graph = helper.make_graph([matmul_node], 'test_graph_1', [A, B], [C])
    model = helper.make_model(graph)
    model = helper.make_model(graph, **{'opset_imports': [helper.make_opsetid('', 13)]})
    return model

def build_model_with_gather():
    b_value = np.random.randint(2, size=(10)).astype(np.int32)
    B_init = helper.make_tensor('B', TensorProto.INT32, [10], b_value.reshape(10).tolist())
    A = helper.make_tensor_value_info('A', TensorProto.FLOAT, [1, 100, 4])
    D = helper.make_tensor_value_info('D', TensorProto.FLOAT, [100, 4])
    squeeze = onnx.helper.make_node('Squeeze', ['A'], ['D'], name='squeeze')
    B = helper.make_tensor_value_info('B', TensorProto.INT32, [10])
    C = helper.make_tensor_value_info('C', TensorProto.FLOAT, [10, 4])
    node = onnx.helper.make_node('Gather', ['D', 'B'], ['C'], name='gather')
    e_value = np.random.randint(2, size=(10)).astype(np.float32)
    E_init = helper.make_tensor('E', TensorProto.FLOAT, [10, 1], e_value.reshape(10).tolist())
    F = helper.make_tensor_value_info('F', TensorProto.FLOAT, [10, 4])
    add = onnx.helper.make_node('Add', ['C', 'E'], ['F'], name='add')
    graph = helper.make_graph([squeeze, node, add], 'test_graph_1', [A], [F], [B_init, E_init])
    model = helper.make_model(graph, **{'opset_imports': [helper.make_opsetid('', 13)]})
    return model

def build_rename_model():
    b_value = np.random.randint(2, size=(10)).astype(np.int32)
    B_init = helper.make_tensor('B', TensorProto.INT32, [10], b_value.reshape(10).tolist())
    A = helper.make_tensor_value_info('A', TensorProto.FLOAT, [1, 100, 4])
    D = helper.make_tensor_value_info('D', TensorProto.FLOAT, [100, 4])
    squeeze = onnx.helper.make_node('Squeeze', ['A'], ['D'], name='')
    B = helper.make_tensor_value_info('B', TensorProto.INT32, [10])
    C = helper.make_tensor_value_info('C', TensorProto.FLOAT, [10, 4])
    node = onnx.helper.make_node('Gather', ['D', 'B'], ['C'], name='')
    e_value = np.random.randint(2, size=(10)).astype(np.float32)
    E_init = helper.make_tensor('E', TensorProto.FLOAT, [10, 1], e_value.reshape(10).tolist())
    F = helper.make_tensor_value_info('F', TensorProto.FLOAT, [10, 4])
    add = onnx.helper.make_node('Add', ['C', 'E'], ['F'], name='')
    graph = helper.make_graph([squeeze, node, add], 'test_graph_1', [A], [F], [B_init, E_init])
    model = helper.make_model(graph, **{'opset_imports': [helper.make_opsetid('', 13)]})
    return model

class MatmulDataset:
    def __init__(self):
        self.data = []
        self.label = []
        for i in range(3):
            self.data.append([np.random.randn(1,5,5).astype('float32'), 
                              np.random.randn(1,5,1).astype('float32')])
            self.label.append(np.random.randn(1,5,1).astype('float32'))

    def __getitem__(self, idx):
        return self.data[idx], self.label[idx]

    def __len__(self):
        return len(self.data)

class TestAdaptorONNXRT(unittest.TestCase):

    mb_v2_export_path = "mb_v2.onnx"
    mb_v2_model = torchvision.models.mobilenet_v2()
    rn50_export_path = "rn50.onnx"
    rn50_model = torchvision.models.resnet50()

    datasets = DATASETS('onnxrt_qlinearops')
    cv_dataset = datasets['dummy'](shape=(100, 3, 224, 224), low=0., high=1., label=True)
    cv_dataloader = DATALOADERS['onnxrt_qlinearops'](cv_dataset)
    
    ir3_dataset = datasets['dummy'](shape=(10, 2048), low=0., high=1., label=True)
    ir3_dataloader = DATALOADERS['onnxrt_qlinearops'](ir3_dataset)

    gather_dataset = DATASETS('onnxrt_qlinearops')['dummy'](shape=(5, 100, 4), label=True)
    gather_dataloader = DATALOADERS['onnxrt_qlinearops'](gather_dataset)

    rename_dataloader = gather_dataloader

    matmul_dataset = MatmulDataset()
    matmul_dataloader = DATALOADERS['onnxrt_qlinearops'](matmul_dataset)

    @classmethod
    def setUpClass(self):
        build_rename_yaml()
        build_static_yaml()
        build_dynamic_yaml()
        build_gather_yaml()
        build_non_MSE_yaml()
        build_benchmark_yaml()
        export_onnx_model(self.mb_v2_model, self.mb_v2_export_path)
        self.mb_v2_model = onnx.load(self.mb_v2_export_path)
        export_onnx_model(self.rn50_model, self.rn50_export_path)
        export_onnx_model(self.rn50_model, 'rn50_9.onnx', 9)
        self.rn50_model = onnx.load(self.rn50_export_path)
        self.ir3_model = build_ir3_model()
        self.gather_model = build_model_with_gather()
        self.matmul_model = build_matmul_model()
        self.rename_model = build_rename_model()

    @classmethod
    def tearDownClass(self):
        os.remove("static.yaml")
        os.remove("dynamic.yaml")
        os.remove("non_MSE.yaml")
        os.remove("benchmark.yaml")
        os.remove("gather.yaml")
        os.remove("rename.yaml")
        os.remove("rn50_9.onnx")
        os.remove(self.mb_v2_export_path)
        os.remove(self.rn50_export_path)
        os.remove("best_model.onnx")
        shutil.rmtree("./saved", ignore_errors=True)
        shutil.rmtree("runs", ignore_errors=True)
        shutil.rmtree("./nc_workspace", ignore_errors=True)

    def test_inspect_tensor(self):
        framework_specific_info = {"device": "cpu",
                               "approach": "post_training_static_quant",
                               "random_seed": 1234,
                               "q_dataloader": None,
                               "backend": "qlinearops",
                               "workspace_path": './nc_workspace/{}/{}/'.format(
                                                       'onnxrt',
                                                       'imagenet')}
        framework = "onnxrt_qlinearops"
        adaptor = FRAMEWORKS[framework](framework_specific_info)
        adaptor.inspect_tensor(self.rn50_model, self.cv_dataloader, inspect_type='activation')
        adaptor.inspect_tensor(self.rn50_model, self.cv_dataloader, inspect_type='activation', save_to_disk=True)
        adaptor.inspect_tensor(self.rn50_model, self.cv_dataloader, inspect_type='weight')
        adaptor.inspect_tensor(self.rn50_model, self.cv_dataloader, inspect_type='all')
        adaptor.inspect_tensor(self.rn50_model, self.cv_dataloader, ["Conv_0"], inspect_type='activation')
        op_list = OrderedDict()
        op_list[("Conv_0", "Conv")] = None
        adaptor.inspect_tensor(self.rn50_model, self.cv_dataloader, op_list.keys(), inspect_type='activation')

    def test_set_tensor(self):
        quantizer = Quantization("static.yaml")
        quantizer.calib_dataloader = self.cv_dataloader
        quantizer.eval_dataloader = self.cv_dataloader
        quantizer.model = common.Model(self.mb_v2_model)
        q_model = quantizer()
        framework_specific_info = {"device": "cpu",
                     "approach": "post_training_static_quant",
                     "random_seed": 1234,
                     "q_dataloader": None,
                     "backend": "qlinearops",
                     "workspace_path": './nc_workspace/{}/{}/'.format(
                                             'onnxrt',
                                             'imagenet')}
        framework = "onnxrt_qlinearops"
        adaptor = FRAMEWORKS[framework](framework_specific_info) 
        q_config = {'fused ' + self.mb_v2_model.graph.node[0].name: {'weight': {'granularity': 'per_channel', 'dtype': onnx_proto.TensorProto.INT8, 'scheme': 'sym'}}}
        adaptor.quantize_config = q_config
        version = get_torch_version()
        q_model.save('./best_model.onnx')
        if version >= '1.7':
            adaptor.set_tensor(onnx.load("best_model.onnx"), 
                {self.mb_v2_model.graph.node[0].input[1]: np.random.random([32, 3, 3, 3])})
            adaptor.set_tensor(q_model,
                {self.mb_v2_model.graph.node[0].input[2]: np.random.random([32])})
        else:
            adaptor.set_tensor(onnx.load("best_model.onnx"), 
                {'ConvBnFusion_W_features.0.0.weight': np.random.random([32, 3, 3, 3])})
            adaptor.set_tensor(q_model, {'ConvBnFusion_BN_B_features.0.1.bias': np.random.random([32])})


    def test_adaptor(self):
        for fake_yaml in ["rename.yaml"]:
            quantizer = Quantization(fake_yaml)
            quantizer.calib_dataloader = self.rename_dataloader
            quantizer.eval_dataloader = self.rename_dataloader
            quantizer.model = common.Model(self.rename_model)
            q_model = quantizer()

        for fake_yaml in ["static.yaml", "dynamic.yaml"]:
            quantizer = Quantization(fake_yaml)
            quantizer.calib_dataloader = self.cv_dataloader
            quantizer.eval_dataloader = self.cv_dataloader
            quantizer.model = common.Model(self.rn50_model)
            q_model = quantizer()
            eval_func(q_model)

        import copy
        tmp_model = copy.deepcopy(self.rn50_model)
        tmp_model.opset_import[0].version = 10
        quantizer.model = common.Model(tmp_model)
        q_model = quantizer()
        tmp_model.opset_import.extend([onnx.helper.make_opsetid("", 11)]) 
        quantizer.model = common.Model(tmp_model)
        q_model = quantizer()
        model = onnx.load('rn50_9.onnx')
        quantizer.model = common.Model(model)
        q_model = quantizer()

        framework_specific_info = {"device": "cpu",
                     "approach": "post_training_static_quant",
                     "random_seed": 1234,
                     "q_dataloader": None,
                     "backend": "qlinearops",
                     "workspace_path": './nc_workspace/{}/{}/'.format(
                                             'onnxrt',
                                             'imagenet')}
        framework = "onnxrt_qlinearops"
        adaptor = FRAMEWORKS[framework](framework_specific_info) 
        tune_cfg = {'calib_iteration': 1,
                    'op': {('gather', 'Gather'): {'activation':  {'dtype': ['uint8']},
                                                 'weight': {'dtype': ['uint8']}},
                           ('add', 'Add'): {'activation':  {'dtype': ['uint8']},
                                           'weight': {'dtype': ['int8']}}}}
        adaptor.quantize(tune_cfg, common.Model(self.gather_model), self.gather_dataloader)
        self.assertTrue(len(adaptor.quantizable_ops), 2)
 
        for fake_yaml in ["gather.yaml"]:
            quantizer = Quantization(fake_yaml)
            quantizer.calib_dataloader = self.gather_dataloader
            quantizer.eval_dataloader = self.gather_dataloader
            quantizer.model = common.Model(self.gather_model)
            q_model = quantizer()

            quantizer.model = common.Model(self.matmul_model)
            q_model = quantizer()

            quantizer.eval_dataloader = self.matmul_dataloader
            q_model = quantizer()

            quantizer.calib_dataloader = self.matmul_dataloader
            quantizer.eval_dataloader = self.matmul_dataloader
            quantizer.model = common.Model(self.matmul_model)
            q_model = quantizer()

        for fake_yaml in ["non_MSE.yaml"]:
            quantizer = Quantization(fake_yaml)
            quantizer.calib_dataloader = self.cv_dataloader
            quantizer.eval_dataloader = self.cv_dataloader
            quantizer.model = common.Model(self.mb_v2_model)
            q_model = quantizer()
            eval_func(q_model)

        for fake_yaml in ["static.yaml"]:
            quantizer = Quantization(fake_yaml)
            quantizer.calib_dataloader = self.ir3_dataloader
            quantizer.eval_dataloader = self.ir3_dataloader
            quantizer.model = common.Model(self.ir3_model)
            q_model = quantizer()

            from neural_compressor.utils.utility import recover
            model = recover(self.ir3_model, './nc_workspace/recover/history.snapshot', 0)
            self.assertTrue(model.model == q_model.model)

        for mode in ["performance", "accuracy"]:
            fake_yaml = "benchmark.yaml"
            evaluator = Benchmark(fake_yaml)
            evaluator.b_dataloader = self.cv_dataloader
            evaluator.model = common.Model(self.rn50_model)
            evaluator(mode)


if __name__ == "__main__":
    unittest.main()



#
#  -*- coding: utf-8 -*-
#
#  Copyright (c) 2021 Intel Corporation
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
#

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import logging
import copy
from tensorflow.core.framework import attr_value_pb2
from tensorflow.python.framework import dtypes
from tensorflow.python.framework import tensor_util

from ..graph_base import GraphRewriterBase
from ..graph_util import GraphAnalyzer
from ..graph_util import GraphRewriterHelper as Helper


class BF16Convert(GraphRewriterBase):
    """
    BF16 node convert transformation.
    """

    # TODO: Analysis all of OPs, consider more structures
    # the set of ops that are considered numerically-safe (for execution
    # in bf16), performance-critical, and can run in bf16. These ops are always
    # converted to bf16.
    WHITE_LIST = ["Conv2D",
                  "Conv2DBackpropFilter",
                  "Conv2DBackpropInput",
                  "Conv3D",
                  "Conv3DBackpropFilterV2",
                  "Conv3DBackpropInputV2",
                  "DepthwiseConv2dNative",
                  "DepthwiseConv2dNativeBackpropFilter",
                  "DepthwiseConv2dNativeBackpropInput",
                  "MatMul",
                  "BatchMatMul",
                  "BatchMatMulV2",
                  ]
    # the set of ops that can run in bf16 and are considered numerically-
    # safe (for execution in bf16), but which may be made unsafe by an upstream
    # blacklist op.
    GRAY_LIST = ["Add",
                 "AddN",
                 "AddV2",
                 "AvgPool",
                 "AvgPool3D",
                 "AvgPool3DGrad",
                 "AvgPoolGrad",
                 "BiasAdd",
                 "BiasAddGrad",
                 "BiasAddV1",
                 "FusedBatchNormV2",
                 "FusedBatchNormGradV2",
                #  "FusedBatchNormV3",
                 "FusedBatchNormGradV3",
                 "LeakyRelu",
                 "LeakyReluGrad",
                 "Mul",
                 "Sub",
                 ]
    # the set of ops that are considered numerically-dangerous (i.e.,
    # unsafe for execution in bf16) and whose effects may also be observed in
    # downstream nodes (e.g. for f16, in Exp -> Add, the Add is unsafe due to
    # the Exp).
    BLACK_LIST = ["Exp",
                  "Expm1",
                  "L2Loss",
                  "Mean",
                  "Pow",
                  "SaveV2",
                  "Softmax",
                  "SoftmaxCrossEntropyWithLogits",
                  "SparseSoftmaxCrossEntropyWithLogits",
                  "Sum",
                  ]
    # the set of ops that do not have numerically-significant effects
    # (i.e., they are always considered safe for execution in bf16 precision), and
    # can run in bf16.
    CLEAR_LIST = ["Concat",
                  "ConcatV2",
                  "Enter",
                  "EnsureShape",
                  "Equal",
                  "Exit",
                  "ExpandDims",
                  "Identity",
                  "MaxPool",
                  "MaxPool3D",
                  "MaxPool3DGrad",
                  "MaxPoolGrad",
                  "MaxPoolV2",
                  "Maximum",
                  "Merge",
                  "NextIteration",
                  "PreventGradient",
                  "Relu",
                  "Relu6",
                  "Relu6Grad",
                  "ReluGrad",
                  "Reshape",
                  # "Select",
                  # "SelectV2",
                  # "Shape",
                  # "ShapeN",
                  # "Slice",
                  # "Split",
                  # "SplitV",
                  # "Squeeze",
                  # "StopGradient",
                  # "Switch",
                  # "Transpose",
                  # "ZerosLike",
                  ]

    def __init__(self,
                 model,
                 fp32_ops=[],
                 bf16_ops=[]):
        super().__init__(model)

        self.cur_graph = GraphAnalyzer()
        self.cur_graph.graph = self.model
        self.fp32_ops = fp32_ops
        self.bf16_ops = bf16_ops
        self.converted_ops = []

    def _bf16_convert(self, bf16_node_name):
        self.converted_ops.append(bf16_node_name)
        bf16_node_detail = self.cur_graph.node_name_details[bf16_node_name]
        bf16_node = bf16_node_detail.node
        bf16_node_inputs = [i for i in list(bf16_node.input) if not i.startswith('^')]

        if 'T' in bf16_node.attr and bf16_node.attr['T'] != \
                attr_value_pb2.AttrValue(type=dtypes.float32.as_datatype_enum) and \
                    bf16_node.op != 'Dequantize':
            return
        for each_input in bf16_node_inputs:
            each_input_detail = self.cur_graph.node_name_details[Helper.node_name_from_input(
                each_input)]

            each_input_node = each_input_detail.node
            # Const + Cast => Const optimization

            if each_input_node.op == "Const":
                if each_input_node.attr["dtype"] == attr_value_pb2.AttrValue(
                        type=dtypes.float32.as_datatype_enum):
                    fp32_value = tensor_util.MakeNdarray(each_input_node.attr.get('value').tensor)
                    Helper.set_attr_dtype(each_input_node, "dtype", dtypes.bfloat16)
                    each_input_node.attr['value'].CopyFrom(attr_value_pb2.AttrValue(
                        tensor=tensor_util.make_tensor_proto(
                            fp32_value, dtypes.bfloat16, fp32_value.shape)))
                    self.converted_ops.append(each_input)
            elif 'T' in each_input_node.attr and each_input_node.attr['T'] != \
                attr_value_pb2.AttrValue(type=dtypes.float32.as_datatype_enum) and \
                    each_input_node.op != 'Dequantize':
                continue
            # Cast + Cast => O optimization
            elif (each_input_node.op == "Cast" and
                  each_input_node.attr["SrcT"] == attr_value_pb2.AttrValue(
                      type=dtypes.bfloat16.as_datatype_enum)):
                cast_input_name = each_input_node.input[0]
                for index, input_name in enumerate(bf16_node.input):
                    if input_name == each_input_node.name:
                        bf16_node.input[index] = cast_input_name
                self.cur_graph.node_name_details[cast_input_name].outputs.append(bf16_node_name)
                if len(each_input_detail.outputs) == 1:
                    self.cur_graph.remove_node(each_input)
                    self.cur_graph.node_name_details[cast_input_name].outputs.remove(each_input)
            elif (each_input not in self.fp32_ops + self.converted_ops and
                    each_input_node.op in BF16Convert.WHITE_LIST + \
                    BF16Convert.GRAY_LIST + BF16Convert.CLEAR_LIST and
                    len(each_input_detail.outputs) == 1):
                self._bf16_convert(each_input)
                # TODO: Consider multi-output case
            elif each_input in self.converted_ops:
                pass
            else:
                if each_input + "_FP32toBF16" not in list(self.cur_graph.node_name_details.keys()):
                    input_cast_node = Helper.create_node(
                        "Cast", each_input.replace(':', '__') + "_FP32toBF16", [each_input])
                    Helper.set_attr_dtype(input_cast_node, "DstT", dtypes.bfloat16)
                    Helper.set_attr_dtype(input_cast_node, "SrcT", dtypes.float32)
                    Helper.set_attr_bool(input_cast_node, "Truncate", False)
                    self.cur_graph.add_node(input_cast_node, each_input, [bf16_node_name])
                else:
                    input_cast_node = self.cur_graph.node_name_details[each_input +
                                                                       "_FP32toBF16"].node
                    for index, input_name in enumerate(bf16_node.input):
                        if Helper.node_name_from_input(input_name) == each_input:
                            bf16_node.input[index] = input_cast_node.name
                    self.cur_graph.node_name_details[input_cast_node.name].outputs.append(
                        bf16_node_name)

        # TODO: Need consider different op type
        Helper.set_attr_dtype(bf16_node, "T", dtypes.bfloat16)

        bf16_node_outputs = copy.deepcopy(bf16_node_detail.outputs)
        for each_output in bf16_node_outputs:
            each_output_detail = self.cur_graph.node_name_details[each_output]
            each_output_node = each_output_detail.node
            # Need consider output node op type

            if (each_output_node.op == "Cast" and
                    each_output_node.attr["DstT"] == attr_value_pb2.AttrValue(
                    type=dtypes.bfloat16.as_datatype_enum)):
                for cast_output in each_output_detail.outputs:
                    cast_output_node = self.cur_graph.node_name_details[cast_output].node
                    for index, input_name in enumerate(cast_output_node.input):
                        if each_output == input_name:
                            cast_output_node.input[index] = bf16_node.name
                bf16_node_detail.outputs.remove(each_output)
                bf16_node_detail.outputs.extend(each_output_detail.outputs)
                self.cur_graph.remove_node(each_output)
            elif (each_output not in self.fp32_ops + self.converted_ops and
                    each_output_node.op in BF16Convert.WHITE_LIST + \
                    BF16Convert.GRAY_LIST + BF16Convert.CLEAR_LIST) :
                # TODO: Consider multi node inputs case, check others inputs whether
                # converted to BF16
                self._bf16_convert(each_output)
            elif each_output in self.converted_ops:
                pass
            else:
                if bf16_node_name + \
                        "_BF16toFP32" not in list(self.cur_graph.node_name_details.keys()):
                    output_cast_node = Helper.create_node(
                        "Cast", bf16_node_name + "_BF16toFP32", [bf16_node_name])
                    Helper.set_attr_dtype(output_cast_node, "DstT", dtypes.float32)
                    Helper.set_attr_dtype(output_cast_node, "SrcT", dtypes.bfloat16)
                    Helper.set_attr_bool(output_cast_node, "Truncate", False)
                    self.cur_graph.add_node(output_cast_node, bf16_node_name, [each_output])
                else:
                    output_cast_node = self.cur_graph.node_name_details[bf16_node_name +
                                                                        "_BF16toFP32"].node
                    for index, input_name in enumerate(each_output_node.input):
                        if bf16_node_name == input_name:
                            each_output_node.input[index] = output_cast_node.name
                    self.cur_graph.node_name_details[bf16_node_name +
                                                     "_BF16toFP32"].outputs.append(each_output)

        return

    def _model_bf16_convert(self):
        logging.debug("start convert bf16 graph")
        self.cur_graph.parse_graph()
        for bf16_node_name in set(self.bf16_ops):
            if bf16_node_name not in self.converted_ops:
                self._bf16_convert(bf16_node_name)

    def _convert_bf16_bnv3(self, input_graph):
        g = GraphAnalyzer()
        g.graph = input_graph
        graph_info = g.parse_graph()

        bn_pattern= [['FusedBatchNormV3']]

        target_nodes = g.query_fusion_pattern_nodes(bn_pattern)
        for i in target_nodes:
            node_name = i[0]
            bn_node = graph_info[node_name].node
            bn_node_outputs = graph_info[node_name].outputs

            x_input_name = bn_node.input[0]
            x_input_node =graph_info[x_input_name].node
            if x_input_node.op == 'Cast' and x_input_node.attr['SrcT'].type == dtypes.bfloat16 \
                and x_input_node.attr['DstT'].type == dtypes.float32:
                g.remove_node_with_single_input_output(x_input_name)
                Helper.set_attr_dtype(bn_node, "T", dtypes.bfloat16)
                output_y_node = graph_info[bn_node_outputs[0]].node
                if output_y_node.attr["T"].type == dtypes.float32:
                    input_cast_node = Helper.create_node(
                        "Cast", node_name.replace(':', '__') + "_BF16toFP32", [node_name])
                    Helper.set_attr_dtype(input_cast_node, "SrcT", dtypes.bfloat16)
                    Helper.set_attr_dtype(input_cast_node, "DstT", dtypes.float32)
                    Helper.set_attr_bool(input_cast_node, "Truncate", False)
                    g.add_node(input_cast_node, node_name, bn_node_outputs)
                else:
                    g.remove_node_with_single_input_output(bn_node_outputs[0])

        return g.dump_graph()

    def do_transformation(self):
        """
        Execute BF16 convert.
        :return: Transformed graph
        """
        self._model_bf16_convert()
        converted_graph_def = self._convert_bf16_bnv3(self.cur_graph.dump_graph())

        converted_graph_def.library.CopyFrom(self.model.library)
        return converted_graph_def



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2021 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import onnx
import onnxruntime
from onnx.numpy_helper import to_array
import numpy as np
import re
from collections import namedtuple, OrderedDict
from neural_compressor.utils import logger
from .ops.tensor import Tensor
from . import graph_utils as util


def get_node_children_names(model, node):
    """Get the node's output nodes' name in the graph
    Args:
        model: neural_compressor ONNXModel
        node: NodeProto in onnx model
    Returns:
        outputs: names list
    """

    output_nodes = model.get_children(node)
    outputs = [node.name for node in output_nodes]
    return outputs


def get_initializer_children_names(model, initializer):
    """Get the initializer's output nodes' name in the graph
    Args:
        model: neural_compressor ONNXModel
        initializer: initializer in onnx model
    Returns:
        outputs: names list
    """

    output_nodes = model.find_nodes_by_initializer(model.graph(), initializer)
    outputs = [node.name for node in output_nodes]
    return outputs


def graph_node_names_details(model):
    """Parse the graph nodes ans get the graph_nodes_dict.
    Be used for Grpah class with cerating a new graph.
    The node_name is the key, node in value is for getting the Const
    tensor value and the input_tensor source op; output_names in value
    is the node ouput name list; outputs in value is for output_tensor dest op
    Args:
        model: neural_compressor ONNXModel
    Returns:
        node_names_details: the graph node info dict

    """

    node_details = namedtuple('node_details', ['node', 'outputs'])
    node_names_details = {}
    for initializer in model.initializer():
        initializer_name = initializer.name
        each_node = node_details(node=initializer, outputs=[])
        if initializer_name not in node_names_details:
            each_node.outputs.extend(get_initializer_children_names(model, initializer))
            node_names_details[initializer_name] = each_node
    for node in model.nodes():
        node_name = node.name
        output_names = node.output
        # onnx output has different name from node name
        for output_name in output_names:
            if output_name not in node_names_details:
                node_names_details[output_name] = node_name
        each_node = node_details(node=node, outputs=[])
        if node_name not in node_names_details:
            each_node.outputs.extend(get_node_children_names(model, node))
            node_names_details[node_name] = each_node
    for graph_input in model.graph().input:
        outputs = []
        node_name = graph_input.name
        for k, v in node_names_details.items():
            try:
                if node_name in v.node.input:
                    outputs.append(k)
            except BaseException:
                continue
        each_node = node_details(node=graph_input, outputs=outputs)
        # if node_name not in node_names_details:
        node_names_details[node_name] = each_node

    return node_names_details


def change_num_name(tensor_name):
    # for number string
    try:
        if str(int(tensor_name)) == tensor_name:
            tensor_name += '_tensor'
    except BaseException:
        pass
    return tensor_name


def bias_to_int32(bias_node, a_scale, b_scale):
    """convert the int8 bias to int32 bias
    Args:
        bias_node: bias_add in graph (from onnx framework)
        a_scale: matmul node input matrice a scale tensor
        b_scale: matmul node input matrice b scale tensor
        model: Grpah class
    Returns:
        int32 bias numpy array

    fp32_bias = (int8_bias - int8_bias_zero_point) * int8_bias_scale
    int32_bias = fp32_bias / (a_scale * b_scale)
    """
    dtype_list = [np.int8, np.uint8]
    input_tensors = bias_node.input_tensors
    if input_tensors[0].source_op == []:
        int8_bias = input_tensors[0].data
        int8_bias_scale = input_tensors[1].data
        int8_bias_zero_point = input_tensors[2].data
    else:
        int8_bias = input_tensors[3].data
        int8_bias_scale = input_tensors[4].data
        int8_bias_zero_point = input_tensors[5].data

    if int8_bias.dtype not in dtype_list:
        return
    int8_bias = int8_bias.astype(np.int32)
    fp32_bias = ((int8_bias - int8_bias_zero_point) * int8_bias_scale).astype(np.float32)
    a_scale = float(a_scale)
    b_scale = float(b_scale)
    int32_bias = np.round((fp32_bias / (a_scale * b_scale))).astype(np.int32)

    return int32_bias


def onnx_extract_operator(node, model, nodes_dict):
    """decorate the operator in onnx
    Args:
        node: NodeProto
        model: neural_compressor ONNXModel
        nodes_dict: dict, return value from graph_node_names_details
        tf_dtypes: dict, for get the dtype string

    Returns:
        op_type: node op type
        input_tensors: Tensor list, contains the node input tensors info
        output_tensors: Tensor list, contains the node output tensor info
    """
    op_type = node.op_type
    input_tensors = []
    output_tensors = []

    """ input_tensors
    each input_tensor has its own soure op, but same dest op
    so both have single string
    """
    input_names = []
    # name list
    input_tensor_names = node.input
    for input_tensor_name in input_tensor_names:
        origin_tensor_name, input_tensor_name = util.names_from_input(input_tensor_name)
        try:
            pre_node = nodes_dict[nodes_dict[origin_tensor_name]].node
        except BaseException:
            pre_node = nodes_dict[origin_tensor_name].node
        
        data = None
        if pre_node in model.initializer():
            data = to_array(pre_node)
        else:
            if (pre_node not in model.graph().input) and (pre_node.op_type == 'Constant'):
                data = to_array(pre_node.attribute[0].t)
        if isinstance(data, np.ndarray):
            dtype = util.get_data_dtype(data)
            shape = list(data.shape) if data.shape != () else [1]
            input_tensor = Tensor(name=input_tensor_name,
                                  source_op=[],
                                  dest_op=[node.name],
                                  shape=shape,
                                  data=data,
                                  dtype=dtype
                                  )
            input_tensors.append(input_tensor)

        else:
            input_tensor = Tensor(name=input_tensor_name,
                                  source_op=[pre_node.name],
                                  dest_op=[node.name],
                                  shape=None,
                                  data=None,
                                  dtype=None
                                  )
            input_tensors.append(input_tensor)
        input_names.append(node.name)

    """ output_tensors
    in onnx, NodeProto has the output attribute
    """
    output_tensor_names = node.output
    for output_tensor_name in output_tensor_names:
        output_tensor_name = util.names_from_input(output_tensor_name)[1]
        output_tensor = Tensor(name=output_tensor_name,
                               source_op=[node.name],
                               dest_op=nodes_dict[node.name].outputs,
                               shape=None,
                               data=None,
                               dtype=None
                               )

        output_tensors.append(output_tensor)

    return op_type, input_tensors, output_tensors


ONNX_DTYPE_ID = {1: 'float32',
                7: 'int32',
                9: 'bool',}



#
#  -*- coding: utf-8 -*-
#
import shutil
import unittest
import os
import yaml
import tensorflow as tf
from tensorflow.python.platform import gfile
from tensorflow.python.framework import graph_util
from tensorflow.python.framework import tensor_util
from neural_compressor.adaptor.tf_utils.util import disable_random

def build_fake_yaml():
    fake_yaml = '''
        model:
          name: fake_yaml
          framework: tensorflow
          inputs: input
          outputs: op_to_store
        device: cpu
        evaluation:
          accuracy:
            metric:
              topk: 1
        tuning:
            accuracy_criterion:
              relative: 0.0001
            workspace:
              path: saved
        '''
    y = yaml.load(fake_yaml, Loader=yaml.SafeLoader)
    with open('fake_yaml.yaml', "w", encoding="utf-8") as f:
        yaml.dump(y, f)
    f.close()

def build_fake_yaml_2():
    fake_yaml = '''
        model:
          name: fake_yaml
          framework: tensorflow
          inputs: input
          outputs: op_to_store
        device: cpu
        graph_optimization:
          precisions: [bf16]
        evaluation:
          accuracy:
            metric:
              topk: 1
        tuning:
            accuracy_criterion:
              relative: 0.0001
            workspace:
              path: saved
        '''
    y = yaml.load(fake_yaml, Loader=yaml.SafeLoader)
    with open('fake_yaml_2.yaml', "w", encoding="utf-8") as f:
        yaml.dump(y, f)
    f.close()

class TestTensorflowRecover(unittest.TestCase):
    @classmethod
    def setUpClass(self):
        build_fake_yaml()

    @classmethod
    def tearDownClass(self):
        os.remove('fake_yaml.yaml')
        os.remove('test.pb')
        shutil.rmtree('./saved', ignore_errors=True)

    @disable_random()
    def test_tensorflow_recover(self):

        x = tf.compat.v1.placeholder(tf.float32, [1, 56, 56, 16], name="input")
        top_relu = tf.nn.relu(x)
        paddings = tf.constant([[0, 0], [1, 1], [1, 1], [0, 0]])
        x_pad = tf.pad(top_relu, paddings, "CONSTANT")
        conv_weights = tf.compat.v1.get_variable("weight", [3, 3, 16, 16],
                                                 initializer=tf.compat.v1.random_normal_initializer())
        conv_weights_2 = tf.compat.v1.get_variable("weight_2", [3, 8, 16, 16],
                                                   initializer=tf.compat.v1.random_normal_initializer())
        conv = tf.nn.conv2d(x_pad, conv_weights, strides=[1, 2, 2, 1], padding="VALID")
        relu = tf.nn.relu(conv)

        max_pool = tf.nn.max_pool(relu, ksize=1, strides=[1, 2, 2, 1], padding="SAME")
        conv_bias = tf.compat.v1.get_variable("bias", [16],
                                              initializer=tf.compat.v1.random_normal_initializer())
        conv_1 = tf.nn.conv2d(max_pool, conv_weights_2, strides=[
                              1, 2, 2, 1], padding="VALID", name='conv1_3')
        conv_bias = tf.math.add(conv_1, conv_bias)
        relu6 = tf.nn.relu6(conv_bias, name='op_to_store')

        out_name = relu6.name.split(':')[0]
        with tf.compat.v1.Session() as sess:
            sess.run(tf.compat.v1.global_variables_initializer())
            constant_graph = graph_util.convert_variables_to_constants(
                sess=sess,
                input_graph_def=sess.graph_def,
                output_node_names=[out_name])
            with gfile.GFile('./test.pb', "wb") as f:
                f.write(constant_graph.SerializeToString())

            from neural_compressor.experimental import Quantization, common
            quantizer = Quantization("./fake_yaml.yaml")
            dataset = quantizer.dataset('dummy', shape=(100, 56, 56, 16), label=True)
            quantizer.calib_dataloader = common.DataLoader(dataset)
            quantizer.model = constant_graph
            q_model = quantizer()

            from neural_compressor.utils.utility import recover
            recover_model = recover('./test.pb', './saved/history.snapshot', 0)

            q_model_const_value = {}
            for node in q_model.graph_def.node:
                if node.op == "Const":
                    tensor_value = tensor_util.MakeNdarray(node.attr["value"].tensor)
                    if not tensor_value.shape:
                        q_model_const_value[node.name] = tensor_value
            for node in recover_model.graph_def.node:
                if node.op == "Const":
                    tensor_value = tensor_util.MakeNdarray(node.attr["value"].tensor)
                    if node.name in q_model_const_value:
                        self.assertEqual(tensor_value, q_model_const_value[node.name])

class TestTensorflowRecoverForceBF16(unittest.TestCase):
    @classmethod
    def setUpClass(self):
        os.environ['FORCE_BF16'] = '1'
        build_fake_yaml_2()

    @classmethod
    def tearDownClass(self):
        del os.environ['FORCE_BF16']
        os.remove('fake_yaml_2.yaml')
        os.remove('test.pb')
        shutil.rmtree('./saved', ignore_errors=True)

    @disable_random()
    def test_tensorflow_recover_bf16(self):
        x = tf.compat.v1.placeholder(tf.float32, [1, 56, 56, 16], name="input")
        top_relu = tf.nn.relu(x)
        paddings = tf.constant([[0, 0], [1, 1], [1, 1], [0, 0]])
        x_pad = tf.pad(top_relu, paddings, "CONSTANT")
        conv_weights = tf.compat.v1.get_variable("weight", [3, 3, 16, 16],
                                                 initializer=tf.compat.v1.random_normal_initializer())
        conv_weights_2 = tf.compat.v1.get_variable("weight_2", [3, 8, 16, 16],
                                                   initializer=tf.compat.v1.random_normal_initializer())
        conv = tf.nn.conv2d(x_pad, conv_weights, strides=[1, 2, 2, 1], padding="VALID")
        relu = tf.nn.relu(conv)

        max_pool = tf.nn.max_pool(relu, ksize=1, strides=[1, 2, 2, 1], padding="SAME")
        conv_bias = tf.compat.v1.get_variable("bias", [16],
                                              initializer=tf.compat.v1.random_normal_initializer())
        conv_1 = tf.nn.conv2d(max_pool, conv_weights_2, strides=[
                              1, 2, 2, 1], padding="VALID", name='conv1_3')
        conv_bias = tf.math.add(conv_1, conv_bias)
        relu6 = tf.nn.relu6(conv_bias, name='op_to_store')

        out_name = relu6.name.split(':')[0]
        with tf.compat.v1.Session() as sess:
            sess.run(tf.compat.v1.global_variables_initializer())
            constant_graph = graph_util.convert_variables_to_constants(
                sess=sess,
                input_graph_def=sess.graph_def,
                output_node_names=[out_name])
            with gfile.GFile('./test.pb', "wb") as f:
                f.write(constant_graph.SerializeToString())

            from neural_compressor.experimental import Quantization, common
            quantizer = Quantization("./fake_yaml_2.yaml")
            dataset = quantizer.dataset('dummy', shape=(100, 56, 56, 16), label=True)
            quantizer.calib_dataloader = common.DataLoader(dataset)
            quantizer.model = constant_graph
            q_model = quantizer()
            found_cast_op = False

            from neural_compressor.utils.utility import recover
            recover_model = recover('./test.pb', './saved/history.snapshot', 0)

            q_model_const_value = {}
            for node in q_model.graph_def.node:
                if node.op == "Const":
                    tensor_value = tensor_util.MakeNdarray(node.attr["value"].tensor)
                    if not tensor_value.shape:
                        q_model_const_value[node.name] = tensor_value
            for node in recover_model.graph_def.node:
                if node.op == 'Cast':
                    found_cast_op = True
                    continue
                if node.op == "Const":
                    tensor_value = tensor_util.MakeNdarray(node.attr["value"].tensor)
                    if node.name in q_model_const_value:
                        self.assertEqual(tensor_value, q_model_const_value[node.name])

            self.assertEqual(found_cast_op, True)


if __name__ == "__main__":
    unittest.main()


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2021 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import onnx
from onnx.numpy_helper import to_array
from neural_compressor.utils import logger
import numpy as np
from ..graph.graph import Graph
from ..ops.op import OPERATORS
from ..onnx_utils import graph_node_names_details
from ..graph_utils import names_from_input


class ONNXExtractor(object):
    """
    Decorate the node in model.graph_def, and the new node has the attributes like input_tensors
    and output_tensors, these tensors record the source/dest op name. All of these nodes 
    (in a list) will compose a graph, which is Graph class, as the return object.
    Args:
        model: neural_compressor TensorflowBaseModel
    Return:
        Graph: Graph class, the new graph object

    """
    @classmethod
    def __call__(self, model):
        # onnx_consts = model.initializer()
        graph_nodes_dict = graph_node_names_details(model)
        logger.info('Start to extarct onnx model ops...')
        new_graph = Graph()
        new_graph.framework = 'onnxruntime'
        for graph_input in model.graph().input:
            op_type = 'ONNXINPUT'
            new_node = OPERATORS[op_type]()
            new_node.extract('onnxruntime', graph_input, model, graph_nodes_dict)
            new_graph.insert_nodes(len(new_graph.nodes), [new_node])
        for node in model.nodes():
            op_type = node.op_type
            if op_type == 'Constant':
                continue
            else:
                if op_type in OPERATORS.keys():
                    new_node = OPERATORS[op_type]()
                    new_node.extract('onnxruntime', node, model, graph_nodes_dict)
                    new_graph.insert_nodes(len(new_graph.nodes), [new_node])
                else:
                    raise ValueError('the {} operation does not support now...'.format(op_type))

        return new_graph



# -*- coding: utf-8 -*-
# Copyright (c) 2021 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tuning config test."""

import unittest

from neural_compressor.ux.utils.exceptions import ClientErrorException
from neural_compressor.ux.utils.workload.tuning import (
    AccCriterion,
    ExitPolicy,
    Strategy,
    Tuning,
    Workspace,
)


class TestStrategyConfig(unittest.TestCase):
    """Strategy config tests."""

    def __init__(self, *args: str, **kwargs: str) -> None:
        """Strategy config test constructor."""
        super().__init__(*args, **kwargs)

    def test_strategy_constructor(self) -> None:
        """Test Strategy config constructor."""
        data = {
            "name": "mse",
            "accuracy_weight": 0.5,
            "latency_weight": 1.0,
            "sigopt_api_token": "abcdefg",
        }
        strategy = Strategy(data)

        self.assertEqual(strategy.name, "mse")
        self.assertEqual(strategy.sigopt_api_token, "abcdefg")
        self.assertEqual(strategy.accuracy_weight, 0.5)
        self.assertEqual(strategy.latency_weight, 1.0)

    def test_strategy_constructor_defaults(self) -> None:
        """Test Strategy config constructor defaults."""
        strategy = Strategy()

        self.assertEqual(strategy.name, "basic")
        self.assertIsNone(strategy.sigopt_api_token)
        self.assertIsNone(strategy.accuracy_weight)
        self.assertIsNone(strategy.latency_weight)

    def test_strategy_serializer(self) -> None:
        """Test Strategy config serializer."""
        data = {
            "name": "bayesian",
            "sigopt_api_token": "abcdefg",
            "accuracy_weight": 0.5,
            "latency_weight": 1.0,
        }
        strategy = Strategy(data)
        result = strategy.serialize()

        self.assertDictEqual(
            result,
            {
                "name": "bayesian",
                "sigopt_api_token": "abcdefg",
                "accuracy_weight": 0.5,
                "latency_weight": 1.0,
            },
        )


class TestAccCriterionConfig(unittest.TestCase):
    """AccCriterion config tests."""

    def __init__(self, *args: str, **kwargs: str) -> None:
        """Acc Criterion config test constructor."""
        super().__init__(*args, **kwargs)

    def test_acc_critetion_constructor(self) -> None:
        """Test AccCriterion config constructor."""
        data = {
            "relative": 0.01,
            "absolute": 0.02,
        }
        acc_critetion = AccCriterion(data)

        self.assertEqual(acc_critetion.relative, 0.01)
        self.assertEqual(acc_critetion.absolute, 0.02)

    def test_acc_critetion_constructor_defaults(self) -> None:
        """Test AccCriterion config constructor defaults."""
        acc_critetion = AccCriterion()

        self.assertIsNone(acc_critetion.relative)
        self.assertIsNone(acc_critetion.absolute)

    def test_acc_critetion_serializer(self) -> None:
        """Test AccCriterion config serializer."""
        data = {
            "relative": 0.01,
            "absolute": 0.02,
        }
        acc_critetion = AccCriterion(data)
        result = acc_critetion.serialize()

        self.assertDictEqual(
            result,
            {
                "relative": 0.01,
                "absolute": 0.02,
            },
        )


class TestExitPolicyConfig(unittest.TestCase):
    """ExitPolicy config tests."""

    def __init__(self, *args: str, **kwargs: str) -> None:
        """Exit Policy config test constructor."""
        super().__init__(*args, **kwargs)

    def test_exit_policy_constructor(self) -> None:
        """Test ExitPolicy config constructor."""
        data = {
            "timeout": 60,
            "max_trials": 200,
        }
        exit_policy = ExitPolicy(data)

        self.assertEqual(exit_policy.timeout, 60)
        self.assertEqual(exit_policy.max_trials, 200)

    def test_exit_policy_constructor_defaults(self) -> None:
        """Test ExitPolicy config constructor defaults."""
        exit_policy = ExitPolicy()

        self.assertIsNone(exit_policy.timeout)
        self.assertIsNone(exit_policy.max_trials)

    def test_exit_policy_serializer(self) -> None:
        """Test ExitPolicy config serializer."""
        data = {
            "timeout": 60,
            "max_trials": 200,
        }
        exit_policy = ExitPolicy(data)
        result = exit_policy.serialize()

        self.assertDictEqual(
            result,
            {
                "timeout": 60,
                "max_trials": 200,
            },
        )


class TestWorkspaceConfig(unittest.TestCase):
    """Workspace config tests."""

    def __init__(self, *args: str, **kwargs: str) -> None:
        """Workspace config test constructor."""
        super().__init__(*args, **kwargs)

    def test_exit_policy_constructor(self) -> None:
        """Test Workspace config constructor."""
        data = {
            "path": "/path/to/workspace",
            "resume": "/path/to/snapshot/file",
        }
        workspace = Workspace(data)

        self.assertEqual(workspace.path, "/path/to/workspace")
        self.assertEqual(workspace.resume, "/path/to/snapshot/file")

    def test_workspace_constructor_defaults(self) -> None:
        """Test Workspace config constructor defaults."""
        workspace = Workspace()

        self.assertIsNone(workspace.path)
        self.assertIsNone(workspace.resume)

    def test_workspace_serializer(self) -> None:
        """Test Workspace config serializer."""
        data = {
            "path": "/path/to/workspace",
            "resume": "/path/to/snapshot/file",
            "additional_field": 1,
        }
        workspace = Workspace(data)
        result = workspace.serialize()

        self.assertDictEqual(
            result,
            {
                "path": "/path/to/workspace",
                "resume": "/path/to/snapshot/file",
            },
        )


class TestTuningConfig(unittest.TestCase):
    """Tuning config tests."""

    def __init__(self, *args: str, **kwargs: str) -> None:
        """Tuning config test constructor."""
        super().__init__(*args, **kwargs)

    def test_tuning_constructor(self) -> None:
        """Test Tuning config constructor."""
        data = {
            "strategy": {
                "name": "mse",
                "accuracy_weight": 0.5,
                "latency_weight": 1.0,
            },
            "accuracy_criterion": {
                "relative": 0.01,
                "absolute": 0.02,
            },
            "objective": "performance",
            "exit_policy": {
                "timeout": 60,
                "max_trials": 200,
            },
            "random_seed": 12345,
            "tensorboard": True,
            "workspace": {
                "path": "/path/to/workspace",
                "resume": "/path/to/snapshot/file",
            },
        }
        tuning = Tuning(data)

        self.assertIsNotNone(tuning.strategy)
        self.assertEqual(tuning.strategy.name, "mse")
        self.assertEqual(tuning.strategy.accuracy_weight, 0.5)
        self.assertEqual(tuning.strategy.latency_weight, 1.0)

        self.assertIsNotNone(tuning.accuracy_criterion)
        self.assertEqual(tuning.accuracy_criterion.relative, 0.01)
        self.assertEqual(tuning.accuracy_criterion.absolute, 0.02)

        self.assertEqual(tuning.objective, "performance")

        self.assertIsNotNone(tuning.exit_policy)
        self.assertEqual(tuning.exit_policy.timeout, 60)
        self.assertEqual(tuning.exit_policy.max_trials, 200)

        self.assertEqual(tuning.random_seed, 12345)

        self.assertTrue(tuning.tensorboard)

        self.assertIsNotNone(tuning.workspace)
        self.assertEqual(tuning.workspace.path, "/path/to/workspace")
        self.assertEqual(tuning.workspace.resume, "/path/to/snapshot/file")

    def test_tuning_constructor_defaults(self) -> None:
        """Test Tuning config constructor defaults."""
        tuning = Tuning()

        self.assertIsNotNone(tuning.strategy)
        self.assertEqual(tuning.strategy.name, "basic")
        self.assertIsNone(tuning.strategy.accuracy_weight)
        self.assertIsNone(tuning.strategy.latency_weight)

        self.assertIsNotNone(tuning.accuracy_criterion)
        self.assertIsNone(tuning.accuracy_criterion.relative)
        self.assertIsNone(tuning.accuracy_criterion.absolute)

        self.assertIsNone(tuning.objective)
        self.assertIsNone(tuning.exit_policy)
        self.assertIsNone(tuning.random_seed)
        self.assertIsNone(tuning.tensorboard)

        self.assertIsNotNone(tuning.workspace)
        self.assertIsNone(tuning.workspace.path)
        self.assertIsNone(tuning.workspace.resume)

    def test_set_timeout(self) -> None:
        """Test setting timeout in Tuning config."""
        tuning = Tuning()
        tuning.set_timeout(10)
        self.assertIsNotNone(tuning.exit_policy)
        self.assertEqual(tuning.exit_policy.timeout, 10)

    def test_set_timeout_with_exit_policy(self) -> None:
        """Test overwriting timeout in Tuning config."""
        tuning = Tuning(
            {
                "exit_policy": {
                    "timeout": 60,
                },
            },
        )
        self.assertIsNotNone(tuning.exit_policy)
        self.assertEqual(tuning.exit_policy.timeout, 60)

        tuning.set_timeout(10)
        self.assertIsNotNone(tuning.exit_policy)
        self.assertEqual(tuning.exit_policy.timeout, 10)

    def test_set_timeout_from_string(self) -> None:
        """Test overwriting timeout in Tuning config."""
        tuning = Tuning()
        tuning.set_timeout("10")
        self.assertIsNotNone(tuning.exit_policy)
        self.assertEqual(tuning.exit_policy.timeout, 10)

    def test_set_timeout_negative(self) -> None:
        """Test overwriting timeout in Tuning config."""
        tuning = Tuning()
        with self.assertRaises(ClientErrorException):
            tuning.set_timeout(-1)

    def test_set_timeout_invalid_string(self) -> None:
        """Test overwriting timeout in Tuning config."""
        tuning = Tuning()
        with self.assertRaises(ClientErrorException):
            tuning.set_timeout("abc")

    def test_set_max_trials(self) -> None:
        """Test setting max_trials in Tuning config."""
        tuning = Tuning()
        tuning.set_max_trials(10)
        self.assertIsNotNone(tuning.exit_policy)
        self.assertEqual(tuning.exit_policy.max_trials, 10)

    def test_set_max_trials_with_exit_policy(self) -> None:
        """Test overwriting max_trials in Tuning config."""
        tuning = Tuning(
            {
                "exit_policy": {
                    "max_trials": 60,
                },
            },
        )
        self.assertIsNotNone(tuning.exit_policy)
        self.assertEqual(tuning.exit_policy.max_trials, 60)

        tuning.set_max_trials(10)
        self.assertIsNotNone(tuning.exit_policy)
        self.assertEqual(tuning.exit_policy.max_trials, 10)

    def test_set_max_trials_from_string(self) -> None:
        """Test overwriting max_trials in Tuning config."""
        tuning = Tuning()
        tuning.set_max_trials("10")
        self.assertIsNotNone(tuning.exit_policy)
        self.assertEqual(tuning.exit_policy.max_trials, 10)

    def test_set_max_trials_negative(self) -> None:
        """Test overwriting max_trials in Tuning config."""
        tuning = Tuning()
        with self.assertRaises(ClientErrorException):
            tuning.set_max_trials(-1)

    def test_set_max_trials_invalid_string(self) -> None:
        """Test overwriting max_trials in Tuning config."""
        tuning = Tuning()
        with self.assertRaises(ClientErrorException):
            tuning.set_max_trials("abc")

    def test_set_performance_only(self) -> None:
        """Test setting performance_only in Tuning config."""
        tuning = Tuning()
        tuning.set_performance_only(True)
        self.assertIsNotNone(tuning.exit_policy)
        self.assertTrue(tuning.exit_policy.performance_only)

    def test_set_performance_only_with_exit_policy(self) -> None:
        """Test overwriting performance_only in Tuning config."""
        tuning = Tuning(
            {
                "exit_policy": {
                    "performance_only": "true",
                },
            },
        )
        self.assertIsNotNone(tuning.exit_policy)
        self.assertTrue(tuning.exit_policy.performance_only)

        tuning.set_performance_only(False)
        self.assertIsNotNone(tuning.exit_policy)
        self.assertFalse(tuning.exit_policy.performance_only)

    def test_set_performance_only_from_string(self) -> None:
        """Test overwriting performance_only in Tuning config."""
        tuning = Tuning()
        tuning.set_performance_only("true")
        self.assertIsNotNone(tuning.exit_policy)
        self.assertTrue(tuning.exit_policy.performance_only)

    def test_set_performance_only_invalid_string(self) -> None:
        """Test overwriting performance_only in Tuning config."""
        tuning = Tuning()
        with self.assertRaises(ClientErrorException):
            tuning.set_performance_only("abc")

    def test_set_random_seed(self) -> None:
        """Test setting random_seed in Tuning config."""
        tuning = Tuning()
        tuning.set_random_seed(123456)
        self.assertEqual(tuning.random_seed, 123456)

    def test_set_random_seed_from_string(self) -> None:
        """Test setting random_seed from string in Tuning config."""
        tuning = Tuning()
        tuning.set_random_seed("123456")
        self.assertEqual(tuning.random_seed, 123456)

    def test_set_random_seed_invalid_string(self) -> None:
        """Test setting random_seed from invalid string in Tuning config."""
        tuning = Tuning()
        with self.assertRaises(ClientErrorException):
            tuning.set_random_seed("abc")

    def test_tuning_serializer(self) -> None:
        """Test Tuning config serializer."""
        data = {
            "strategy": {
                "name": "mse",
                "accuracy_weight": 0.5,
                "latency_weight": 1.0,
            },
            "accuracy_criterion": {
                "relative": 0.01,
                "absolute": 0.02,
            },
            "objective": "performance",
            "exit_policy": {
                "timeout": 60,
                "max_trials": 200,
            },
            "random_seed": 12345,
            "tensorboard": True,
            "workspace": {
                "path": "/path/to/workspace",
                "resume": "/path/to/snapshot/file",
            },
            "additional_field": {"key": "val"},
        }
        tuning = Tuning(data)
        result = tuning.serialize()

        self.assertDictEqual(
            result,
            {
                "strategy": {
                    "name": "mse",
                    "accuracy_weight": 0.5,
                    "latency_weight": 1.0,
                },
                "accuracy_criterion": {
                    "relative": 0.01,
                    "absolute": 0.02,
                },
                "objective": "performance",
                "exit_policy": {
                    "timeout": 60,
                    "max_trials": 200,
                },
                "random_seed": 12345,
                "tensorboard": True,
                "workspace": {
                    "path": "/path/to/workspace",
                    "resume": "/path/to/snapshot/file",
                },
            },
        )


if __name__ == "__main__":
    unittest.main()



import os
import shutil
import unittest
import subprocess

import torch
import torchvision
import torch.nn as nn
import horovod.torch as hvd

from neural_compressor.data import DATASETS
from neural_compressor.experimental.data.dataloaders.pytorch_dataloader import PyTorchDataLoader

def build_fake_py():
    fake_py = """
import os
import shutil
import unittest

import torch
import torchvision
import torch.nn as nn
import horovod.torch as hvd

from neural_compressor.data import DATASETS
from neural_compressor.experimental.data.dataloaders.pytorch_dataloader import PyTorchDataLoader



class TestPruning(unittest.TestCase):

    model = torchvision.models.resnet18()

    def test_pruning_internal(self):
        from neural_compressor.experimental import Pruning, common
        prune = Pruning('fake.yaml')

        prune.model = common.Model(self.model)
        _ = prune()
        print('rank {} in size {}'.format(hvd.rank(), hvd.size()))

if __name__ == "__main__":
    unittest.main()
    """
    with open('fake.py', 'w', encoding="utf-8") as f:
        f.write(fake_py)

def build_fake_yaml():
    fake_yaml = """
    model:
      name: imagenet_prune
      framework: pytorch

    pruning:
      train:
        start_epoch: 0
        end_epoch: 4
        iteration: 10
        dataloader:
          batch_size: 30
          distributed: True
          dataset:
            dummy:
              shape: [128, 3, 224, 224]
              label: True
        optimizer:
          SGD:
            learning_rate: 0.1
            momentum: 0.1
            nesterov: True
            weight_decay: 0.1
        criterion:
          CrossEntropyLoss:
            reduction: sum
      approach:
        weight_compression:
          initial_sparsity: 0.0
          target_sparsity: 0.97
          start_epoch: 0
          end_epoch: 4
          pruners:
            - !Pruner
                start_epoch: 1
                end_epoch: 3
                prune_type: basic_magnitude
                names: ['layer1.0.conv1.weight']

            - !Pruner
                target_sparsity: 0.6
                prune_type: gradient_sensitivity
                update_frequency: 2
                names: ['layer1.0.conv2.weight']
    evaluation:
      accuracy:
        metric:
          topk: 1
        dataloader:
          distributed: True
          batch_size: 30
          dataset:
            dummy:
              shape: [128, 3, 224, 224]
              label: True
    """
    with open('fake.yaml', 'w', encoding="utf-8") as f:
        f.write(fake_yaml)


class TestDistributed(unittest.TestCase):
    model = torchvision.models.resnet18()
    @classmethod
    def setUpClass(cls):
        build_fake_yaml()
        build_fake_py()

    @classmethod
    def tearDownClass(cls):
        os.remove('fake.yaml')
        os.remove('fake.py')
        shutil.rmtree('./saved', ignore_errors=True)
        shutil.rmtree('runs', ignore_errors=True)


    def test_distributed(self):
        distributed_cmd = 'horovodrun -np 2 python fake.py'
        p = subprocess.Popen(distributed_cmd, preexec_fn=os.setsid, stdout=subprocess.PIPE,
                             stderr=subprocess.PIPE, shell=True) # nosec
        try:
            out, error = p.communicate()
            import re
            matches = re.findall(r'.*rank ([01]) in size 2.*', out.decode('utf-8'))
            assert '0' in matches
            assert '1' in matches
        except KeyboardInterrupt:
            os.killpg(os.getpgid(p.pid), signal.SIGKILL)
            assert 0

    def test_single_node(self):
        from neural_compressor.experimental import Pruning, common
        prune = Pruning('fake.yaml')

        prune.model = common.Model(self.model)
        _ = prune()
        # assert hvd hook is registered. pruner has 2 pre_epoch_begin hooks: hvd and prune
        assert len(prune.hooks_dict['pre_epoch_begin'])==2

if __name__ == "__main__":
    unittest.main()



import os
import shutil
import unittest

import torch
import torchvision
import torch.nn as nn
from torch.quantization.quantize_fx import convert_fx, prepare_qat_fx

from neural_compressor.data import DATASETS
from neural_compressor.experimental.data.dataloaders.pytorch_dataloader import PyTorchDataLoader
from neural_compressor.experimental.scheduler import Scheduler

fake_yaml = """
model:
  name: imagenet_prune
  framework: pytorch

pruning:
  approach:
    weight_compression:
      initial_sparsity: 0.0
      target_sparsity: 0.97
      start_epoch: 0
      end_epoch: 4
      pruners:
        - !Pruner
            start_epoch: 1
            end_epoch: 3
            prune_type: basic_magnitude
            names: ['layer1.0.conv1.weight']

        - !Pruner
            target_sparsity: 0.6
            prune_type: basic_magnitude
            update_frequency: 2
            names: ['layer1.0.conv2.weight']
evaluation:
  accuracy:
    metric:
      topk: 1
"""

fake2_yaml = """
model:
  name: imagenet_qat
  framework: pytorch

quantization:
  approach: quant_aware_training

evaluation:
  accuracy:
    metric:
      topk: 1
tuning:
  accuracy_criterion:
    relative:  0.01
  exit_policy:
    timeout: 0
  random_seed: 9527
"""


def build_fake_yaml():
    with open('fake.yaml', 'w', encoding="utf-8") as f:
        f.write(fake_yaml)


def build_fake_yaml2():
    with open('fake2.yaml', 'w', encoding="utf-8") as f:
        f.write(fake2_yaml)

def build_fx_fake_yaml():
    fx_fake_yaml = fake_yaml.replace('pytorch', 'pytorch_fx')
    with open('fx_fake.yaml', 'w', encoding="utf-8") as f:
        f.write(fx_fake_yaml)


def build_fx_fake_yaml2():
    fx_fake2_yaml = fake2_yaml.replace('pytorch', 'pytorch_fx')
    with open('fx_fake2.yaml', 'w', encoding="utf-8") as f:
        f.write(fx_fake2_yaml)


class TestPruning(unittest.TestCase):

    model = torchvision.models.resnet18()
    q_model = torchvision.models.quantization.resnet18()

    @classmethod
    def setUpClass(cls):
        build_fake_yaml()
        build_fake_yaml2()
        build_fx_fake_yaml()
        build_fx_fake_yaml2()

    @classmethod
    def tearDownClass(cls):
        os.remove('fake.yaml')
        os.remove('fake2.yaml')
        os.remove('fx_fake.yaml')
        os.remove('fx_fake2.yaml')
        shutil.rmtree('./saved', ignore_errors=True)
        shutil.rmtree('runs', ignore_errors=True)

    def test_oneshot(self):
        from neural_compressor.experimental import Pruning, common, Quantization
        datasets = DATASETS('pytorch')
        dummy_dataset = datasets['dummy'](shape=(100, 3, 224, 224), low=0., high=1., label=True)
        dummy_dataloader = PyTorchDataLoader(dummy_dataset)
        self.q_model.fuse_model()
        prune = Pruning('./fake.yaml')
        quantizer = Quantization('./fake2.yaml')
        scheduler = Scheduler()
        scheduler.model = common.Model(self.q_model)
        combination = scheduler.combine(prune, quantizer)

        def train_func_for_nc(model):
            epochs = 5
            iters = 3
            criterion = nn.CrossEntropyLoss()
            optimizer = torch.optim.SGD(model.parameters(), lr=0.0001)
            model.qconfig = torch.quantization.get_default_qat_qconfig('fbgemm')
            torch.quantization.prepare_qat(model, inplace=True)
            for nepoch in range(epochs):
                model.train()
                cnt = 0
                combination.on_epoch_begin(nepoch)
                for image, target in dummy_dataloader:
                    combination.on_batch_begin(cnt)
                    print('.', end='')
                    cnt += 1
                    output = model(image)
                    loss = criterion(output, target)
                    optimizer.zero_grad()
                    loss.backward()
                    optimizer.step()
                    combination.on_batch_end()
                    if cnt >= iters:
                        break
                combination.on_epoch_end()
            torch.quantization.convert(model, inplace=True)

        combination.train_func = train_func_for_nc
        combination.eval_dataloader = dummy_dataloader
        combination.train_dataloader = dummy_dataloader
        scheduler.append(combination)
        opt_model = scheduler()

        conv_weight = opt_model.model.layer1[0].conv1.weight().dequantize()
        self.assertAlmostEqual((conv_weight == 0).sum().item() / conv_weight.numel(),
                               0.97,
                               delta=0.01)
        self.assertEqual(combination.__repr__().lower(), 'combination of pruning,quantization')

    def test_oneshot_fx(self):
        from neural_compressor.experimental import Pruning, common, Quantization
        datasets = DATASETS('pytorch_fx')
        dummy_dataset = datasets['dummy'](shape=(100, 3, 224, 224), low=0., high=1., label=True)
        dummy_dataloader = PyTorchDataLoader(dummy_dataset)
        prune = Pruning('./fx_fake.yaml')
        quantizer = Quantization('./fx_fake2.yaml')
        scheduler = Scheduler()
        scheduler.model = common.Model(self.model)
        combination = scheduler.combine(prune, quantizer)

        def train_func_for_nc(model):
            epochs = 5
            iters = 3
            criterion = nn.CrossEntropyLoss()
            optimizer = torch.optim.SGD(model.parameters(), lr=0.0001)
            qconfig = {"":torch.quantization.get_default_qat_qconfig('fbgemm')}
            model = prepare_qat_fx(model, qconfig)
            # TODO: For test here, needs to bypass this in fx q_function
            combination.model.model = model
            for nepoch in range(epochs):
                model.train()
                cnt = 0
                combination.on_epoch_begin(nepoch)
                for image, target in dummy_dataloader:
                    combination.on_batch_begin(cnt)
                    print('.', end='')
                    cnt += 1
                    output = model(image)
                    loss = criterion(output, target)
                    optimizer.zero_grad()
                    loss.backward()
                    optimizer.step()
                    combination.on_batch_end()
                    if cnt >= iters:
                        break
                combination.on_epoch_end()
            model = convert_fx(model)
            return model

        combination.train_func = train_func_for_nc
        combination.eval_dataloader = dummy_dataloader
        combination.train_dataloader = dummy_dataloader
        scheduler.append(combination)
        opt_model = scheduler()

        conv_weight = dict(opt_model.model.layer1.named_modules())['0'].conv1.weight().dequantize()
        self.assertAlmostEqual((conv_weight == 0).sum().item() / conv_weight.numel(),
                               0.97,
                               delta=0.01)
        self.assertEqual(combination.__repr__().lower(), 'combination of pruning,quantization')


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2021 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from neural_compressor.model.model import MODELS, get_model_fwk_name, get_model_type
from neural_compressor.utils.utility import LazyImport
onnx = LazyImport('onnx')
 
class Loader(object):
    def __call__(self, model):
        framework = get_model_fwk_name(model)
        if framework =='tensorflow':
            model_type = get_model_type(model)
            model =MODELS[framework](model_type, model)
        if framework =='onnxruntime':
            model = onnx.load(model)
            model =MODELS[framework](model)
        return model



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Copyright (c) 2021 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""WSGI Web Server."""
import sys

import gevent.monkey

already_patched = gevent.monkey.is_module_patched("threading")
if not already_patched:
    can_patch_ssl = "ssl" not in sys.modules
    gevent.monkey.patch_all(ssl=can_patch_ssl)

from neural_compressor.ux.utils.environment import Environment  # noqa: E402
from neural_compressor.ux.utils.exceptions import NotFoundException  # noqa: E402
from neural_compressor.ux.utils.logger import change_log_level  # noqa: E402
from neural_compressor.ux.web.configuration import Configuration  # noqa: E402
from neural_compressor.ux.web.server import run_server  # noqa: E402


def main() -> None:
    """Get parameters and initialize server."""
    try:
        configuration = Configuration()
    except NotFoundException as e:
        print(str(e))
        sys.exit(1)

    prepare_environment()

    change_log_level(configuration.log_level)
    print(
        "Intel(r) Neural Compressor Bench Server started.\n"
        "Setup port forwarding from "
        f"your local port {configuration.gui_port} to "
        f"{configuration.server_port} on this machine.\n"
        f"Then open address {configuration.get_url()}",
    )

    run_server(configuration)


def prepare_environment() -> None:
    """Prepare environment for Intel® Neural Compressor Bench."""
    environment = Environment()
    environment.ensure_workdir_exists_and_writeable()
    environment.migrate_workloads_list()
    environment.migrate_workloads()
    environment.clean_workloads_wip_status()


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
# Copyright (c) 2021 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tuning History test."""

import unittest
from unittest.mock import MagicMock, patch

from neural_compressor.ux.components.optimization.tuning_history import tuning_history
from neural_compressor.ux.utils.exceptions import NotFoundException


@patch("sys.argv", ["neural_compressor_bench.py", "-p5000"])
class TestTuningHistory(unittest.TestCase):
    """Test Tuning History."""

    @patch("neural_compressor.ux.utils.templates.workdir.Workdir.get_workload_data")
    def test_get_history_snapshot_when_no_workload_found(
        self,
        mocked_get_workload_data_by_id: MagicMock,
    ) -> None:
        """Test get_history_snapshot."""
        mocked_get_workload_data_by_id.return_value = {}

        with self.assertRaisesRegex(
            NotFoundException,
            "Unable to find workload with id: requested workload id",
        ):
            tuning_history("requested workload id")

        mocked_get_workload_data_by_id.assert_called_with("requested workload id")


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2021 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from abc import abstractmethod

# for complicated pattern, use a several lists and tuple to represent the main computation flow
# and sub-graphs
# for example, layer_norm pattern in bert_large:
# pattern: [ [(0, 'Mean'), (1, 'SquaredDifference'), (2, 'Mean'), (3, 'AddV2'), (4, 'Rsqrt'),
#             (5, 'Mul'),
#             (7 ,'Mul'), (8, 'Sub'), (9, 'AddV2')], [(5, 'Mul'), (6, 'Mul'), (9, 'AddV2')] ]
# the pattern has order, but need connections check cause of the staggered edges
# for detailed info, please see search_pattern func in converter/graph_utils.py
# supported patterns should have order
supported_patterns = [
    'InputFile',
    'InputData',
    'LayerNorm',
    'WordEmbeddings',
    'TokenTypeEmbeddings',
    'PositionEmbeddings',
    'PaddingSequence',
    'MatMulWithBias',
    'LastLayerShape',
    'AttentionReshape',
    'QKVReshape',
    'TransposeBatchMatMul',
    'Gelu',
    'MatMulWithBiasGelu',
    'MatMulWithBiasAdd',
    'AddEmbeddings',
    'MatMulWithBiasTanh',
    'MatMulWithBiasRelu',
    'LayerNormWithReduceMean',
    'StartEndLogits',
    'OutputData',
]

PATTERNS = {}


def pattern_registry(pattern_type):
    """The class decorator used to register all Algorithm subclasses.

    Args:
        cls (class): The class of register.
        pattern_type (str): The pattern registration name

    Returns:
        cls: The class of register.
    """
    def decorator_pattern(cls):
        if pattern_type in PATTERNS:
            raise ValueError('Cannot have two patterns with the same name')
        PATTERNS[pattern_type] = cls
        return cls

    return decorator_pattern


class Pattern(object):
    @abstractmethod
    def __call__(self, model, *args, **kwargs):
        raise NotImplementedError



import unittest
import tensorflow as tf
from tensorflow.python.framework import graph_util

class TestConvertLayout(unittest.TestCase):
    def test_convert_layout(self):
        tf.compat.v1.disable_eager_execution()
        x = tf.compat.v1.placeholder(tf.float32, [1, 10, 10, 3], name="input")
        conv_weights = tf.compat.v1.get_variable("weight", [3, 3, 10, 3],
                            initializer=tf.compat.v1.random_normal_initializer())
        conv = tf.nn.conv2d(x, conv_weights, strides=[1, 2, 2, 1], padding="VALID", 
                                data_format='NCHW')
        relu = tf.nn.relu(conv, name='relu')
        out_name = relu.name.split(':')[0]
        with tf.compat.v1.Session() as sess:
            sess.run(tf.compat.v1.global_variables_initializer())
            output_graph_def = graph_util.convert_variables_to_constants(
                sess=sess,
                input_graph_def=sess.graph_def,
                output_node_names=[out_name])

        from neural_compressor.adaptor.tf_utils.graph_rewriter.generic import convert_layout
        convert = convert_layout.ConvertLayoutOptimizer(output_graph_def, [out_name])
        convert_graph = convert.do_transformation()
        for node in convert_graph.node:
            if node.op == 'Conv2D' and 'data_format' in node.attr:
                if tf.version.VERSION >= '2.4.0':
                    self.assertEqual(node.attr['data_format'].s, b'NHWC')
                else:
                    self.assertEqual(node.attr['data_format'].s, b'NCHW')

if __name__ == "__main__":
    unittest.main()



"""Tests for quantization"""
import numpy as np
import unittest
import os
import shutil
import yaml
import tensorflow as tf
from tensorflow.core.framework import attr_value_pb2
from tensorflow.core.framework import graph_pb2
from tensorflow.core.framework import node_def_pb2
from tensorflow.python.framework import tensor_util
from tensorflow.python.framework import dtypes

def build_fake_yaml():
    fake_yaml = '''
        model:
          name: fake_yaml
          framework: tensorflow
          inputs: x
          outputs: op_to_store
        device: cpu
        evaluation:
          accuracy:
            metric:
              topk: 1
        tuning:
            strategy:
              name: bayesian
            exit_policy:
              max_trials: 1
            accuracy_criterion:
              relative: 0.01
            workspace:
              path: saved
        '''
    y = yaml.load(fake_yaml, Loader=yaml.SafeLoader)
    with open('fake_yaml.yaml',"w",encoding="utf-8") as f:
        yaml.dump(y,f)
    f.close()

def build_fake_yaml2():
    fake_yaml = '''
        model:
          name: fake_yaml
          framework: tensorflow
          inputs: input
          outputs: conv3
        device: cpu
        quantization:
          calibration:
            sampling_size: 10, 20
          op_wise: {
                     \"conv1\": {
                       \"activation\":  {\"dtype\": [\"fp32\"]},
                     },
                   }
        evaluation:
          accuracy:
            metric:
              topk: 1
        tuning:
          strategy:
            name: bayesian
          exit_policy:
            timeout: 60
          accuracy_criterion:
            relative: -0.01
          workspace:
            path: saved
        '''
    with open('fake_yaml2.yaml',"w",encoding="utf-8") as f:
        f.write(fake_yaml)
    f.close()

def build_fake_model():
    try:
        graph = tf.Graph()
        graph_def = tf.GraphDef()
        with tf.Session() as sess:
            x = tf.placeholder(tf.float64, shape=(1,3,3,1), name='x')
            y = tf.constant(np.random.random((2,2,1,1)), name='y')
            op = tf.nn.conv2d(input=x, filter=y, strides=[1,1,1,1], padding='VALID', name='op_to_store')

            sess.run(tf.global_variables_initializer())
            constant_graph = tf.graph_util.convert_variables_to_constants(sess, sess.graph_def, ['op_to_store'])

        graph_def.ParseFromString(constant_graph.SerializeToString())
        with graph.as_default():
            tf.import_graph_def(graph_def, name='')
    except:
        graph = tf.Graph()
        graph_def = tf.compat.v1.GraphDef()
        with tf.compat.v1.Session() as sess:
            x = tf.compat.v1.placeholder(tf.float64, shape=(1,3,3,1), name='x')
            y = tf.compat.v1.constant(np.random.random((2,2,1,1)), name='y')
            op = tf.nn.conv2d(input=x, filters=y, strides=[1,1,1,1], padding='VALID', name='op_to_store')

            sess.run(tf.compat.v1.global_variables_initializer())
            constant_graph = tf.compat.v1.graph_util.convert_variables_to_constants(sess, sess.graph_def, ['op_to_store'])

        graph_def.ParseFromString(constant_graph.SerializeToString())
        with graph.as_default():
            tf.import_graph_def(graph_def, name='')
    return graph

def create_test_graph():
    input_node = node_def_pb2.NodeDef()
    input_node.name = "input"
    input_node.op = "Placeholder"
    input_node.attr["dtype"].CopyFrom(attr_value_pb2.AttrValue(
        type=dtypes.float32.as_datatype_enum))

    conv1_weight_node = node_def_pb2.NodeDef()
    conv1_weight_node.name = "conv1_weights"
    conv1_weight_node.op = "Const"
    conv1_weight_value = np.float32(np.abs(np.random.randn(3,3,3,32)))
    conv1_weight_node.attr['dtype'].CopyFrom(attr_value_pb2.AttrValue(type=dtypes.float32.as_datatype_enum))
    conv1_weight_node.attr['value'].CopyFrom(attr_value_pb2.AttrValue(
        tensor=tensor_util.make_tensor_proto(
    conv1_weight_value, conv1_weight_value.dtype.type, conv1_weight_value.shape)))

    conv1_node = node_def_pb2.NodeDef()
    conv1_node.name = "conv1"
    conv1_node.op = "Conv2D"
    conv1_node.attr['T'].CopyFrom(attr_value_pb2.AttrValue(
        type=dtypes.float32.as_datatype_enum))
    conv1_node.input.extend([input_node.name, conv1_weight_node.name])
    conv1_node.attr['strides'].CopyFrom(attr_value_pb2.AttrValue(
        list=attr_value_pb2.AttrValue.ListValue(i=[1,1,1,1])))
    conv1_node.attr['dilations'].CopyFrom(attr_value_pb2.AttrValue(
        list=attr_value_pb2.AttrValue.ListValue(i=[1,1,1,1])))
    conv1_node.attr['padding'].CopyFrom(attr_value_pb2.AttrValue(s=b'SAME'))
    conv1_node.attr['data_format'].CopyFrom(attr_value_pb2.AttrValue(s=b'NHWC'))

    bias_node = node_def_pb2.NodeDef()
    bias_node.name = "conv1_bias"
    bias_node.op = "Const"
    bias_value = np.float32(np.abs(np.random.randn(32)))
    bias_node.attr['dtype'].CopyFrom(attr_value_pb2.AttrValue(type=dtypes.float32.as_datatype_enum))
    bias_node.attr['value'].CopyFrom(attr_value_pb2.AttrValue(tensor=tensor_util.make_tensor_proto(
        bias_value, bias_value.dtype.type, bias_value.shape)))

    bias_add_node = node_def_pb2.NodeDef()
    bias_add_node.name = "conv1_bias_add"
    bias_add_node.op = "BiasAdd"
    bias_add_node.attr['T'].CopyFrom(attr_value_pb2.AttrValue(type=dtypes.float32.as_datatype_enum))
    bias_add_node.input.extend([conv1_node.name, bias_node.name])
    bias_add_node.attr['data_format'].CopyFrom(attr_value_pb2.AttrValue(s=b'NHWC'))

    relu_node = node_def_pb2.NodeDef()
    relu_node.op = "Relu"
    relu_node.name = "relu"
    relu_node.attr['T'].CopyFrom(attr_value_pb2.AttrValue(type=dtypes.float32.as_datatype_enum))
    relu_node.input.extend([bias_add_node.name])

    conv2_weight_node = node_def_pb2.NodeDef()
    conv2_weight_node.name = "conv2_weights"
    conv2_weight_node.op = "Const"
    conv2_weight_value = np.float32(np.abs(np.random.randn(3,3,32,32)))
    conv2_weight_node.attr['dtype'].CopyFrom(attr_value_pb2.AttrValue(type=dtypes.float32.as_datatype_enum))
    conv2_weight_node.attr['value'].CopyFrom(attr_value_pb2.AttrValue(
        tensor=tensor_util.make_tensor_proto(
    conv2_weight_value, conv2_weight_value.dtype.type, conv2_weight_value.shape)))

    conv2_node = node_def_pb2.NodeDef()
    conv2_node.name = "conv2"
    conv2_node.op = "Conv2D"
    conv2_node.attr['T'].CopyFrom(attr_value_pb2.AttrValue(
        type=dtypes.float32.as_datatype_enum))
    conv2_node.input.extend([relu_node.name, conv2_weight_node.name])
    conv2_node.attr['strides'].CopyFrom(attr_value_pb2.AttrValue(
        list=attr_value_pb2.AttrValue.ListValue(i=[1,1,1,1])))
    conv2_node.attr['dilations'].CopyFrom(attr_value_pb2.AttrValue(
        list=attr_value_pb2.AttrValue.ListValue(i=[1,1,1,1])))
    conv2_node.attr['padding'].CopyFrom(attr_value_pb2.AttrValue(s=b'SAME'))
    conv2_node.attr['data_format'].CopyFrom(attr_value_pb2.AttrValue(s=b'NHWC'))

    bias_node2 = node_def_pb2.NodeDef()
    bias_node2.name = "conv2_bias"
    bias_node2.op = "Const"
    bias_value2 = np.float32(np.abs(np.random.randn(32)))
    bias_node2.attr['dtype'].CopyFrom(attr_value_pb2.AttrValue(type=dtypes.float32.as_datatype_enum))
    bias_node2.attr['value'].CopyFrom(attr_value_pb2.AttrValue(tensor=tensor_util.make_tensor_proto(
        bias_value2, bias_value2.dtype.type, bias_value2.shape)))

    bias_add_node2 = node_def_pb2.NodeDef()
    bias_add_node2.name = "conv2_bias_add"
    bias_add_node2.op = "BiasAdd"
    bias_add_node2.attr['T'].CopyFrom(attr_value_pb2.AttrValue(type=dtypes.float32.as_datatype_enum))
    bias_add_node2.input.extend([conv2_node.name, bias_node2.name])
    bias_add_node2.attr['data_format'].CopyFrom(attr_value_pb2.AttrValue(s=b'NHWC'))

    relu_node2 = node_def_pb2.NodeDef()
    relu_node2.op = "Relu"
    relu_node2.name = "relu2"
    relu_node2.attr['T'].CopyFrom(attr_value_pb2.AttrValue(type=dtypes.float32.as_datatype_enum))
    relu_node2.input.extend([bias_add_node2.name])

    conv3_weight_node = node_def_pb2.NodeDef()
    conv3_weight_node.name = "conv3_weights"
    conv3_weight_node.op = "Const"
    conv3_weight_value = np.float32(np.abs(np.random.randn(3,3,32,32)))
    conv3_weight_node.attr['dtype'].CopyFrom(attr_value_pb2.AttrValue(type=dtypes.float32.as_datatype_enum))
    conv3_weight_node.attr['value'].CopyFrom(attr_value_pb2.AttrValue(
        tensor=tensor_util.make_tensor_proto(
    conv3_weight_value, conv3_weight_value.dtype.type, conv3_weight_value.shape)))

    conv3_node = node_def_pb2.NodeDef()
    conv3_node.name = "conv3"
    conv3_node.op = "Conv2D"
    conv3_node.attr['T'].CopyFrom(attr_value_pb2.AttrValue(
        type=dtypes.float32.as_datatype_enum))
    conv3_node.input.extend([relu_node2.name, conv3_weight_node.name])
    conv3_node.attr['strides'].CopyFrom(attr_value_pb2.AttrValue(
        list=attr_value_pb2.AttrValue.ListValue(i=[1,1,1,1])))
    conv3_node.attr['dilations'].CopyFrom(attr_value_pb2.AttrValue(
        list=attr_value_pb2.AttrValue.ListValue(i=[1,1,1,1])))
    conv3_node.attr['padding'].CopyFrom(attr_value_pb2.AttrValue(s=b'SAME'))
    conv3_node.attr['data_format'].CopyFrom(attr_value_pb2.AttrValue(s=b'NHWC'))

    test_graph = graph_pb2.GraphDef()

    test_graph.node.extend([input_node,
                                 conv1_weight_node,
                                 conv1_node,
                                 bias_node,
                                 bias_add_node,
                                 relu_node,
                                 conv2_weight_node,
                                 conv2_node,
                                 bias_node2,
                                 bias_add_node2,
                                 relu_node2,
                                 conv3_weight_node,
                                 conv3_node,
                                ])
    return test_graph

def objective_func(params):
    return params['x1']**2 + params['x2']

class TestQuantization(unittest.TestCase):

    @classmethod
    def setUpClass(self):
        self.constant_graph = build_fake_model()
        self.test_graph = create_test_graph()
        build_fake_yaml()
        build_fake_yaml2()

    @classmethod
    def tearDownClass(self):
        os.remove('fake_yaml.yaml')
        os.remove('fake_yaml2.yaml')

        shutil.rmtree("saved", ignore_errors=True)

    def test_run_bayesian_one_trial(self):

        from neural_compressor.experimental import Quantization, common
        quantizer = Quantization('fake_yaml.yaml')
        dataset = quantizer.dataset('dummy', shape=(100, 3, 3, 1), label=True)
        quantizer.eval_dataloader = common.DataLoader(dataset)
        quantizer.calib_dataloader = common.DataLoader(dataset)
        quantizer.model = self.constant_graph
        output_graph = quantizer()

    def test_run_bayesian_max_trials(self):

        from neural_compressor.experimental import Quantization, common
        quantizer = Quantization('fake_yaml2.yaml')
        dataset = quantizer.dataset('dummy', shape=(1, 224, 224, 3), label=True)
        quantizer.eval_dataloader = common.DataLoader(dataset)
        quantizer.calib_dataloader = common.DataLoader(dataset)
        quantizer.model = self.test_graph
        output_graph = quantizer()

    def test_bayesian_opt_class(self):
        from neural_compressor.strategy.bayesian import BayesianOptimization
        pbounds = {}
        pbounds['x1'] = (0, 1)
        pbounds['x2'] = (0, 1)
        np.random.seed(9527)
        bayes_opt = BayesianOptimization(pbounds=pbounds,
                                         random_seed=9527)
        for i in range(10):
            params = bayes_opt.gen_next_params()
            try:
                bayes_opt._space.register(params, objective_func(params))
            except KeyError:
                pass
        self.assertTrue(bayes_opt._space.max()['target'] == 2.0)
        self.assertTrue(len(bayes_opt._space.res()) == 8)

if __name__ == "__main__":
    unittest.main()



#
#
#  -*- coding: utf-8 -*-
import unittest
import os
import tensorflow as tf
import yaml

from neural_compressor.adaptor.tf_utils.util import read_graph
from neural_compressor.adaptor.tf_utils.quantize_graph.quantize_graph_for_intel_cpu import QuantizeGraphForIntel
from neural_compressor.adaptor.tensorflow import TensorflowQuery
from neural_compressor.adaptor.tf_utils.util import disable_random
from tensorflow.python.framework import graph_util


def build_fake_yaml():
    fake_yaml = '''
        model:
          name: fake_yaml
          framework: tensorflow
          inputs: input
          outputs: predict
        device: cpu
        quantization:
          model_wise:
            weight:
                granularity: per_tensor
                scheme: sym
                dtype: int8
                algorithm: minmax
        evaluation:
          accuracy:
            metric:
              topk: 1
        tuning:
            strategy:
              name: mse
            accuracy_criterion:
              relative: 0.01
            exit_policy:
              performance_only: True
            workspace:
              path: saved
        '''
    y = yaml.load(fake_yaml, Loader=yaml.SafeLoader)
    with open('fake_yaml.yaml', "w", encoding="utf-8") as f:
        yaml.dump(y, f)
    f.close()


class TestTensorflowConcat(unittest.TestCase):
    mb_model_url = 'https://storage.googleapis.com/intel-optimized-tensorflow/models/v1_8/inceptionv3_fp32_pretrained_model.pb'
    pb_path = '/tmp/.neural_compressor/inceptionv3_fp32.pb'

    @classmethod
    def setUpClass(self):
        if not os.path.exists(self.pb_path):
            os.system(
                "mkdir -p /tmp/.neural_compressor && wget {} -O {} ".format(self.mb_model_url, self.pb_path))
        self.op_wise_sequences = TensorflowQuery(local_config_file=os.path.join(
            os.path.dirname(__file__), "../neural_compressor/adaptor/tensorflow.yaml")).get_eightbit_patterns()
        build_fake_yaml()

    @classmethod
    def tearDownClass(self):
        os.remove('fake_yaml.yaml')

    def test_tensorflow_concat_quantization(self):

        output_graph_def = read_graph(self.pb_path)

        from neural_compressor.experimental import Quantization, common
        quantizer = Quantization('fake_yaml.yaml')
        dataset = quantizer.dataset(
            'dummy', shape=(100, 299, 299, 3), label=True)
        quantizer.eval_dataloader = common.DataLoader(dataset)
        quantizer.calib_dataloader = common.DataLoader(dataset)
        quantizer.model = output_graph_def
        output_graph = quantizer()
        found_quantized_concat_node = False

        target_concat_node_name = 'v0/cg/incept_v3_a0/concat_eightbit_quantized_concatv2'
        from neural_compressor.adaptor.tf_utils.graph_rewriter.graph_util import GraphAnalyzer
        cur_graph = GraphAnalyzer()
        cur_graph.graph = output_graph.graph_def
        graph_info = cur_graph.parse_graph()
        found_quantized_concat_node = target_concat_node_name in graph_info

        self.assertEqual(found_quantized_concat_node, True)
        min_out, max_out = [], []
        for input_conv_name in graph_info[target_concat_node_name].node.input[:4]:
            # print (input_conv_name, graph_info[input_conv_name].node.input)
            min_freezed_out_name = graph_info[input_conv_name].node.input[-2]
            max_freezed_out_name = graph_info[input_conv_name].node.input[-1]
            min_freezed_out_value = (
                graph_info[min_freezed_out_name].node.attr['value'].tensor.float_val)[0]
            max_freezed_out_value = (
                graph_info[max_freezed_out_name].node.attr['value'].tensor.float_val)[0]
            min_out.append(min_freezed_out_value)
            max_out.append(max_freezed_out_value)

        self.assertEqual(len(set(min_out)), 1)
        self.assertEqual(len(set(max_out)), 1)

    @disable_random()
    def test_concat_with_different_input_type(self):
        x = tf.compat.v1.placeholder(
            tf.float32, [1, 128, 128, 16], name="input")
        conv_weights = tf.compat.v1.get_variable("weight", [2, 2, 16, 16],
                                                 initializer=tf.compat.v1.random_normal_initializer())
        conv_bias = tf.compat.v1.get_variable("bias", [16],
                                              initializer=tf.compat.v1.random_normal_initializer())

        x = tf.nn.relu(x)
        sqrt = tf.math.sqrt(x)
        relu_sqrt = tf.nn.relu(sqrt)
        conv = tf.nn.conv2d(relu_sqrt, conv_weights, strides=[
            1, 2, 2, 1], padding="SAME", name='last')
        normed = tf.compat.v1.layers.batch_normalization(conv)

        relu = tf.nn.relu(normed)
        conv1 = tf.nn.conv2d(x, conv_weights, strides=[
            1, 2, 2, 1], padding="SAME", name='last')
        conv_bias = tf.nn.bias_add(conv1, conv_bias)
        concat = tf.concat([relu, conv_bias], 1)
        final_node = tf.nn.relu(concat, name='op_to_store')
        out_name = final_node.name.split(':')[0]
        with tf.compat.v1.Session() as sess:
            sess.run(tf.compat.v1.global_variables_initializer())
            output_graph_def = graph_util.convert_variables_to_constants(
                sess=sess,
                input_graph_def=sess.graph_def,
                output_node_names=[out_name])
            from neural_compressor.experimental import Quantization, common

            quantizer = Quantization('fake_yaml.yaml')
            dataset = quantizer.dataset(
                'dummy', shape=(100, 128, 128, 16), label=True)
            quantizer.calib_dataloader = common.DataLoader(dataset)
            quantizer.eval_dataloader = common.DataLoader(dataset)
            quantizer.model = output_graph_def
            output_graph = quantizer()

            quantized_concat = False
            for i in output_graph.graph_def.node:
              if i.op == 'QuantizedConcatV2':
                  quantized_concat = True
            self.assertEqual(quantized_concat, False)


if __name__ == "__main__":
    unittest.main()
